<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Api;

/**
 * Address CRUD interface.
 * @api
 */
interface AddressRepositoryInterface
{
    /**
     * Save address.
     *
     * @param \Extensa\Speedy\Api\Data\AddressInterface $address
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function save(\Extensa\Speedy\Api\Data\AddressInterface $address);

    /**
     * Retrieve address.
     *
     * @param int $addressId
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getById($addressId);

    /**
     * Retrieve addresses matching the specified criteria.
     *
     * @param \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
     * @return \Extensa\Speedy\Api\Data\AddressSearchResultsInterface
     */
    public function getList(\Magento\Framework\Api\SearchCriteriaInterface $searchCriteria);

    /**
     * Delete address.
     *
     * @param \Extensa\Speedy\Api\Data\AddressInterface $address
     * @return bool true on success
     */
    public function delete(\Extensa\Speedy\Api\Data\AddressInterface $address);

    /**
     * Delete address by ID.
     *
     * @param int $addressId
     * @return bool true on success
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function deleteById($addressId);
}
