<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Api\Data;

/**
 * Speedy Address interface.
 * @api
 */
interface AddressInterface
{
    /**
     * Constants for keys of data array. Identical to the name of the getter in snake case
     */
    const ID                            = 'address_id';
    const ADDRESS_ID                    = 'address_id';
    const CUSTOMER_ID                   = 'customer_id';
    const POSTCODE                      = 'postcode';
    const REQUIRED_POSTCODE             = 'required_postcode';
    const COUNTRY                       = 'country';
    const COUNTRY_ID                    = 'country_id';
    const COUNTRY_NOMENCLATURE          = 'country_nomenclature';
    const ABROAD                        = 'abroad';
    const STATE                         = 'state';
    const STATE_ID                      = 'state_id';
    const REQUIRED_STATE                = 'required_state';
    const CITY                          = 'city';
    const CITY_ID                       = 'city_id';
    const CITY_NOMENCLATURE             = 'city_nomenclature';
    const TAKE_OFFICE                   = 'take_office';
    const OFFICE_ID                     = 'office_id';
    const QUARTER                       = 'quarter';
    const QUARTER_ID                    = 'quarter_id';
    const STREET                        = 'street';
    const STREET_ID                     = 'street_id';
    const STREET_NO                     = 'street_no';
    const BLOCK_NO                      = 'block_no';
    const ENTRANCE_NO                   = 'entrance_no';
    const FLOOR_NO                      = 'floor_no';
    const APARTMENT_NO                  = 'apartment_no';
    const NOTE                          = 'note';
    const ADDRESS_1                     = 'address_1';
    const ADDRESS_2                     = 'address_2';

    /**#@-*/

    /**
     * Get ID
     *
     * @return int
     */
    public function getId();

    /**
     * Get Address ID
     *
     * @return int
     */
    public function getAddressId();

    /**
     * Get Customer ID
     *
     * @return int
     */
    public function getCustomerId();

    /**
     * Get postcode
     *
     * @return string
     */
    public function getPostcode();

    /**
     * Get Required Postcode
     *
     * @return int
     */
    public function getRequiredPostcode();

    /**
     * Get Country
     *
     * @return string
     */
    public function getCountry();

    /**
     * Get Country ID
     *
     * @return int
     */
    public function getCountryId();

    /**
     * Get Country Nomenclature
     *
     * @return string
     */
    public function getCountryNomenclature();

    /**
     * Get abroad
     *
     * @return int
     */
    public function getAbroad();

    /**
     * Get State
     *
     * @return string
     */
    public function getState();

    /**
     * Get State ID
     *
     * @return int
     */
    public function getStateId();

    /**
     * Get Required State
     *
     * @return int
     */
    public function getRequiredState();

    /**
     * Get city
     *
     * @return string
     */
    public function getCity();

    /**
     * Get city ID
     *
     * @return int
     */
    public function getCityId();

    /**
     * Get City Nomenclature
     *
     * @return string
     */
    public function getCityNomenclature();

    /**
     * Get take office
     *
     * @return int
     */
    public function getTakeOffice();

    /**
     * Get office ID
     *
     * @return string
     */
    public function getOfficeId();

    /**
     * Get quarter
     *
     * @return string
     */
    public function getQuarter();

    /**
     * Get quarter ID
     *
     * @return int
     */
    public function getQuarterId();

    /**
     * Get street
     *
     * @return string
     */
    public function getStreet();

    /**
     * Get street ID
     *
     * @return int
     */
    public function getStreetId();

    /**
     * Get street No
     *
     * @return string
     */
    public function getStreetNo();

    /**
     * Get block No
     *
     * @return string
     */
    public function getBlockNo();

    /**
     * Get entrance No
     *
     * @return string
     */
    public function getEntranceNo();

    /**
     * Get floor No
     *
     * @return string
     */
    public function getFloorNo();

    /**
     * Get apartment No
     *
     * @return string
     */
    public function getApartmentNo();

    /**
     * Get note
     *
     * @return string
     */
    public function getNote();

    /**
     * Get address_1
     *
     * @return string
     */
    public function getAddress1();

    /**
     * Get address_2
     *
     * @return string
     */
    public function getAddress2();

    /**
     * Set ID
     *
     * @param int $id
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setId($id);

    /**
     * Set Address ID
     *
     * @param int $addressId
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setAddressId($addressId);

    /**
     * Set Customer ID
     *
     * @param int $customerId
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setCustomerId($customerId);

    /**
     * Set postcode
     *
     * @param string $postcode
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setPostcode($postcode);

    /**
     * Set postcode
     *
     * @param int $requiredPostcode
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setRequiredPostcode($requiredPostcode);

    /**
     * Set Country
     *
     * @param string $country
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setCountry($country);

    /**
     * Set Country ID
     *
     * @param int $countryId
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setCountryId($countryId);

    /**
     * Set Country Nomenclature
     *
     * @param string $countryNomenclature
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setCountryNomenclature($countryNomenclature);

    /**
     * Set Abroad
     *
     * @param int $abroad
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setAbroad($abroad);

    /**
     * Set State
     *
     * @param string $state
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setState($state);

    /**
     * Set State ID
     *
     * @param int $stateId
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setStateId($stateId);

    /**
     * Set Required State
     *
     * @param int $requiredState
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setRequiredState($requiredState);

    /**
     * Set city
     *
     * @param string $city
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setCity($city);

    /**
     * Set City ID
     *
     * @param int $cityId
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setCityId($cityId);

    /**
     * Set City Nomenclature
     *
     * @param string $cityNomenclature
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setCityNomenclature($cityNomenclature);

    /**
     * Set Take Office
     *
     * @param int $takeOffice
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setTakeOffice($takeOffice);

    /**
     * Set Office ID
     *
     * @param int $officeId
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setOfficeId($officeId);

    /**
     * Set quarter
     *
     * @param string $quarter
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setQuarter($quarter);

    /**
     * Set quarter ID
     *
     * @param int $quarterId
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setQuarterId($quarterId);

    /**
     * Set street
     *
     * @param string $street
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setStreet($street);

    /**
     * Set street ID
     *
     * @param int $streetId
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setStreetId($streetId);

    /**
     * Set street Mo
     *
     * @param string $streetNo
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setStreetNo($streetNo);

    /**
     * Set block No
     *
     * @param string $blockNo
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setBlockNo($blockNo);

    /**
     * Set entrance No
     *
     * @param string $entranceNo
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setEntranceNo($entranceNo);

    /**
     * Set floor No
     *
     * @param string $floorNo
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setFloorNo($floorNo);

    /**
     * Set apartment No
     *
     * @param string $apartmentNo
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setApartmentNo($apartmentNo);

    /**
     * Set note
     *
     * @param string $note
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setNote($note);

    /**
     * Set address_1
     *
     * @param string $address_1
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setAddress1($address_1);

    /**
     * Set address_2
     *
     * @param string $address_2
     * @return \Extensa\Speedy\Api\Data\AddressInterface
     */
    public function setAddress2($address_2);
}
