<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Api;

/**
 * Order CRUD interface.
 * @api
 */
interface OrderRepositoryInterface
{
    /**
     * Save order.
     *
     * @param \Extensa\Speedy\Api\Data\OrderInterface $order
     * @return \Extensa\Speedy\Api\Data\OrderInterface
     */
    public function save(\Extensa\Speedy\Api\Data\OrderInterface $order);

    /**
     * Retrieve order.
     *
     * @param int $orderId
     * @return \Extensa\Speedy\Api\Data\OrderInterface
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getById($orderId);

    /**
     * Retrieve cities matching the specified criteria.
     *
     * @param \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
     * @return \Extensa\Speedy\Api\Data\OrderSearchResultsInterface
     */
    public function getList(\Magento\Framework\Api\SearchCriteriaInterface $searchCriteria);

    /**
     * Delete order.
     *
     * @param \Extensa\Speedy\Api\Data\OrderInterface $order
     * @return bool true on success
     */
    public function delete(\Extensa\Speedy\Api\Data\OrderInterface $order);

    /**
     * Delete order by ID.
     *
     * @param int $orderId
     * @return bool true on success
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function deleteById($orderId);
}
