<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Block\Adminhtml\Sales\Order;

class Grid extends \Magento\Backend\Block\Widget\Grid\Extended
{
    /**
     * @var \Extensa\Speedy\Model\ResourceModel\Order\CollectionFactory
     */
    protected $speedyOrderCollectionFactory;

    /**
     * @var \Magento\Sales\Model\Order\Config
     */
    protected $orderConfig;

    /**
     * Grid constructor.
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Backend\Helper\Data $backendHelper
     * @param \Extensa\Speedy\Model\ResourceModel\Order\CollectionFactory $speedyOrderCollectionFactory
     * @param \Magento\Sales\Model\Order\Config $orderConfig
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Backend\Helper\Data $backendHelper,
        \Extensa\Speedy\Model\ResourceModel\Order\CollectionFactory $speedyOrderCollectionFactory,
        \Magento\Sales\Model\Order\Config $orderConfig,
        array $data = []
    ) {
        $this->speedyOrderCollectionFactory = $speedyOrderCollectionFactory;
        $this->orderConfig = $orderConfig;

        parent::__construct($context, $backendHelper, $data);
    }

    /**
     * @return void
     */
    protected function _construct()
    {
        parent::_construct();
        $this->setId('speedyOrderGrid');
        $this->setDefaultSort('shipment_num');
        $this->setDefaultDir('desc');
    }

    /**
     * Prepare collection for grid
     *
     * @return \Magento\Backend\Block\Widget\Grid
     */
    protected function _prepareCollection()
    {
        $collection = $this->speedyOrderCollectionFactory->create();
        $collection->addSalesOrder()->addBolFilter();
        $this->setCollection($collection);

        return parent::_prepareCollection();
    }

    /**
     * Prepare default grid column
     *
     * @return $this
     */
    protected function _prepareColumns()
    {
        parent::_prepareColumns();

        $this->addColumn(
            'bol_id',
            [
                'header'           => __('Bill Of Lading'),
                'type'             => 'string',
                'index'            => 'bol_id',
                'header_css_class' => 'col-id',
                'column_css_class' => 'col-id'
            ]
        );

        $this->addColumn(
            'increment_id',
            [
                'header'           => __('Order #'),
                'type'             => 'string',
                'index'            => 'increment_id',
                'header_css_class' => 'col-id',
                'column_css_class' => 'col-id'
            ]
        );

        $this->addColumn(
            'shipping_name',
            [
                'header'           => __('Sender'),
                'type'             => 'string',
                'index'            => 'shipping_name',
                'header_css_class' => 'col-id',
                'column_css_class' => 'col-id'
            ]
        );

        $this->addColumn(
            'billing_name',
            [
                'header'           => __('Recipient'),
                'type'             => 'string',
                'index'            => 'billing_name',
                'header_css_class' => 'col-id',
                'column_css_class' => 'col-id'
            ]
        );

        $this->addColumn(
            'date_created',
            [
                'header'           => __('Date Created'),
                'type'             => 'datetime',
                'index'            => 'date_created',
                'header_css_class' => 'col-id',
                'column_css_class' => 'col-id'
            ]
        );

        $actions = [];

        if ($this->_authorization->isAllowed('Extensa_Speedy::view_pdf')) {
            $actions[] = [
                'caption' => __('Download'),
                'url'     => ['base' => '*/order/printpdf'],
                'field'   => 'order_id'
            ];
        }

        if ($this->_authorization->isAllowed('Extensa_Speedy::cancel')) {
            $actions[] = [
                'caption' => __('Cancel'),
                'url'     => ['base' => '*/order/cancel'],
                'field'   => 'order_id'
            ];
        }

        if ($this->_authorization->isAllowed('Extensa_Speedy::track')) {
            $actions[] = [
                'caption' => __('Track'),
                'url'     => ['base' => '*/order/track'],
                'field'   => 'order_id'
            ];
        }

        if ($actions) {
            $this->addColumn(
                'action',
                [
                    'header'           => __('Action'),
                    'type'             => 'action',
                    'getter'           => 'getOrderId',
                    'actions'          => $actions,
                    'filter'           => false,
                    'sortable'         => false,
                    'index'            => 'stores',
                    'is_system'        => true,
                    'header_css_class' => 'col-id',
                    'column_css_class' => 'col-id',
                ]
            );
        }

        return $this;
    }

    protected function _prepareMassaction()
    {
        $this->setMassactionIdField('order_id');
        $this->getMassactionBlock()->setFormFieldName('order_ids');
        $this->getMassactionBlock()->setUseSelectAll(false);

        if ($this->_authorization->isAllowed('Extensa_Speedy::cancel')) {
            $this->getMassactionBlock()->addItem('cancel', array(
                'label' => __('Mass cancel'),
                'url'   => $this->getUrl('*/order/massCancel'),
            ));
        }

        return $this;
    }
}
