<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */
namespace Extensa\Speedy\Block\Adminhtml\Sales\Order\View\Tab;

/**
 * Extensa Speedy Create shipment tab
 */
class Create extends \Magento\Sales\Block\Adminhtml\Order\AbstractOrder implements
    \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * @var \Extensa\Speedy\Helper\Data
     */
    protected $dataHelper;

    /**
     * @var \Extensa\Speedy\Helper\Speedy
     */
    protected $speedyHelper;

    /**
     * @var \Extensa\Speedy\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var \Extensa\Speedy\Api\Data\OrderInterfaceFactory
     */
    protected $orderFactory;

    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     * @var \Magento\Framework\Api\FilterBuilder
     */
    protected $filterBuilder;

    /**
     * @var \Magento\Framework\View\Layout $layout
     */
    protected $layout;

    /**
     * @var \Magento\Backend\Model\Auth\Session $authSession
     */
    protected $authSession;

    /**
     * Create constructor.
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Sales\Helper\Admin $adminHelper
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     * @param \Extensa\Speedy\Api\OrderRepositoryInterface $orderRepository
     * @param \Extensa\Speedy\Api\Data\OrderInterfaceFactory $orderFactory
     * @param \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Framework\Api\FilterBuilder $filterBuilder
     * @param \Magento\Framework\View\Layout $layout
     * @param \Magento\Backend\Model\Auth\Session $authSession
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Sales\Helper\Admin $adminHelper,
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Extensa\Speedy\Helper\Speedy $speedyHelper,
        \Extensa\Speedy\Api\OrderRepositoryInterface $orderRepository,
        \Extensa\Speedy\Api\Data\OrderInterfaceFactory $orderFactory,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\FilterBuilder $filterBuilder,
        \Magento\Framework\View\Layout $layout,
        \Magento\Backend\Model\Auth\Session $authSession,
        array $data = []
    ) {
        $this->dataHelper = $dataHelper;
        $this->speedyHelper = $speedyHelper;
        $this->orderRepository = $orderRepository;
        $this->orderFactory = $orderFactory;
        $this->productRepository = $productRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterBuilder = $filterBuilder;
        $this->layout = $layout;
        $this->authSession = $authSession;

        parent::__construct(
            $context,
            $registry,
            $adminHelper,
            $data
        );
    }

    /**
     * Retrieve speedy order model instance
     *
     * @return \Extensa\Speedy\Model\Order
     */
    public function getSpeedyOrder()
    {
        if (!$this->_coreRegistry->registry('current_speedy_order')) {
            $order = [];

            $id = $this->_request->getParam('order_id');
            try {
                $orderIdFilter = $this->filterBuilder
                    ->setField('order_id')
                    ->setValue($id)
                    ->create();

                $searchCriteriaBuilder = $this->searchCriteriaBuilder->addFilters([$orderIdFilter])->create();

                $items = $this->orderRepository->getList($searchCriteriaBuilder)->getItems();

                $order = array_shift($items);

                $this->_coreRegistry->register('current_speedy_order', $order);
            } catch (\Exception $e) {
                $this->_logger->error('Extensa_Speedy - ' . get_class($this) . ' :: ' . $e->getMessage());
            }
        }

        return $this->_coreRegistry->registry('current_speedy_order');
    }

    /**
     * @return $this
     */
    protected function _prepareLayout()
    {
        if ($this->canShowTab()) {
            $parcel_size = $this->layout->createBlock(
                'Extensa\Speedy\Block\Adminhtml\Sales\Order\View\Tab\Create\Parcelsize'
            );

            $speedyOrder = $this->getSpeedyOrder();
            if ($speedyOrder) {
                $speedyOrderData = unserialize($speedyOrder->getDataColumn());

                $parcel_size->setValue($speedyOrderData['parcels_size']);
            }

            $this->setChild('parcel_size', $parcel_size);
        }
        return parent::_prepareLayout();
    }

    public function getSpeedyOrderData()
    {
        $speedyOrderData = [];
        $speedyOrder = $this->getSpeedyOrder();
        $order = $this->getOrder();
        $allItems = $order->getAllItems();

        if ($speedyOrder) {
            $speedyOrderData = unserialize($speedyOrder->getDataColumn());

            $currencyRate = $this->_storeManager->getStore()->getBaseCurrency()->getRate(
                $this->dataHelper->getStoreConfig('currency')
            );

            $total = $order->getBaseGrandTotal() - $order->getBaseShippingAmount();

            if ($total > 0) {
                $total = round(($total * $currencyRate), 2);
            } else {
                $total = 0;
            }

            $speedyOrderData['total'] = $total;

            $insuranceTotal = 0;

            foreach ($allItems as $item) {
                $product = $this->productRepository->getById($item->getProductId());
                if ($product->getId()) {
                    $insuranceTotal += round(($item->getBasePrice() * $item->getQtyOrdered() * $currencyRate), 2);
                }
            }

            $speedyOrderData['insurance_total'] = $insuranceTotal;

            if (!isset($speedyOrderData['content'])) {
                $speedyOrderData['content'] = __('Order #') . ' ' . $order->getIncrementId();
            }

            if (!isset($speedyOrderData['abroad'])) {
                $speedyOrderData['abroad'] = false;
            }

            if (!isset($speedyOrderData['packing'])) {
                $speedyOrderData['packing'] = $this->dataHelper->getStoreConfig('packing');
            }

            if (!isset($speedyOrderData['client_id'])) {
                $speedyOrderData['client_id'] = $this->dataHelper->getStoreConfig('client_id');
            }

            if (!isset($speedyOrderData['payer_type'])) {
                $speedyOrderData['payer_type'] = $this->dataHelper->getStoreConfig('payer_type');
            }

            if (!isset($speedyOrderData['count'])) {
                $speedyOrderData['count'] = $this->dataHelper->getStoreConfig('count');
            } else {
                $speedyOrderData['count'] = 1;
            }

            if (!isset($speedyOrderData['weight'])) {
                $speedyOrderData['weight'] = '';
            }
            if (!isset($speedyOrderData['shipping_method'])) {
                $speedyOrderData['shipping_method'] = '';
            }
            if (!isset($speedyOrderData['method_id'])) {
                $method = explode('_', $order->getShippingMethod());
                $speedyOrderData['method_id'] = end($method);
            }
            if (!isset($speedyOrderData['parcel_size'])) {
                $speedyOrderData['parcel_size'] = '';
            }
            if (!isset($speedyOrderData['deffered_days'])) {
                $speedyOrderData['deffered_days'] = 0;
            }
            if (!isset($speedyOrderData['client_note'])) {
                $speedyOrderData['client_note'] = '';
            }
            if (!isset($speedyOrderData['cod'])) {
                $speedyOrderData['cod'] = true;
            }
            if (!isset($speedyOrderData['convertion_to_win1251'])) {
                $speedyOrderData['convertion_to_win1251'] = false;
            }
            if (!isset($speedyOrderData['option_before_payment'])) {
                $speedyOrderData['option_before_payment'] = $this->dataHelper->getStoreConfig('option_before_payment');
            }
            if (!isset($speedyOrderData['insurance'])) {
                $speedyOrderData['insurance'] = $this->dataHelper->getStoreConfig('insurance');
            }
            if (!isset($speedyOrderData['fragile'])) {
                $speedyOrderData['fragile'] = $this->dataHelper->getStoreConfig('fragile');
            }
            if (!isset($speedyOrderData['postcode'])) {
                $speedyOrderData['postcode'] = '';
            }
            if (!isset($speedyOrderData['city'])) {
                $speedyOrderData['city'] = '';
            }
            if (!isset($speedyOrderData['city_id'])) {
                $speedyOrderData['city_id'] = 0;
            }
            if (!isset($speedyOrderData['city_nomenclature'])) {
                $speedyOrderData['city_nomenclature'] = '';
            }
            if (!isset($speedyOrderData['to_office'])) {
                $speedyOrderData['to_office'] = '';
            }
            if (!isset($speedyOrderData['quarter'])) {
                $speedyOrderData['quarter'] = '';
            }
            if (!isset($speedyOrderData['quarter_id'])) {
                $speedyOrderData['quarter_id'] = 0;
            }
            if (!isset($speedyOrderData['street'])) {
                $speedyOrderData['street'] = '';
            }
            if (!isset($speedyOrderData['street_id'])) {
                $speedyOrderData['street_id'] = 0;
            }
            if (!isset($speedyOrderData['street_no'])) {
                $speedyOrderData['street_no'] = '';
            }
            if (!isset($speedyOrderData['block_no'])) {
                $speedyOrderData['block_no'] = '';
            }
            if (!isset($speedyOrderData['entrance_no'])) {
                $speedyOrderData['entrance_no'] = '';
            }
            if (!isset($speedyOrderData['floor_no'])) {
                $speedyOrderData['floor_no'] = '';
            }
            if (!isset($speedyOrderData['apartment_no'])) {
                $speedyOrderData['apartment_no'] = '';
            }
            if (!isset($speedyOrderData['office_id'])) {
                $speedyOrderData['office_id'] = 0;
            }
            if (!isset($speedyOrderData['note'])) {
                $speedyOrderData['note'] = '';
            }
            if (!isset($speedyOrderData['note'])) {
                $speedyOrderData['note'] = '';
            }
            if (!isset($speedyOrderData['fixed_time'])) {
                $speedyOrderData['fixed_time'] = $this->dataHelper->getStoreConfig('fixed_time');
            }
            if (!isset($speedyOrderData['fixed_time_cb'])) {
                $speedyOrderData['fixed_time_cb'] = false;
            }
            if (!isset($speedyOrderData['fixed_time_hour'])) {
                $speedyOrderData['fixed_time_hour'] = '';
            }
            if (!isset($speedyOrderData['fixed_time_min'])) {
                $speedyOrderData['fixed_time_min'] = '';
            }
            if (!isset($speedyOrderData['country'])) {
                $speedyOrderData['country'] = '';
            }
            if (!isset($speedyOrderData['country_id'])) {
                $speedyOrderData['country_id'] = 0;
            }
            if (!isset($speedyOrderData['country_nomenclature'])) {
                $speedyOrderData['country_nomenclature'] = '';
            }
            if (!isset($speedyOrderData['required_state'])) {
                $speedyOrderData['required_state'] = false;
            }
            if (!isset($speedyOrderData['required_postcode'])) {
                $speedyOrderData['required_postcode'] = false;
            }
            if (!isset($speedyOrderData['active_currency_code'])) {
                $speedyOrderData['active_currency_code'] = '';
            }
            if (!isset($speedyOrderData['state'])) {
                $speedyOrderData['state'] = '';
            }
            if (!isset($speedyOrderData['state_id'])) {
                $speedyOrderData['state_id'] = 0;
            }
            if (!isset($speedyOrderData['address_1'])) {
                $speedyOrderData['address_1'] = '';
            }
            if (!isset($speedyOrderData['address_2'])) {
                $speedyOrderData['address_2'] = '';
            }

            if ($this->dataHelper->getStoreConfig('taking_date')) {
                $speedyOrderData['taking_date'] = date(strtotime('+' . (int)$this->dataHelper->getStoreConfig('taking_date') . ' day', mktime(9, 0, 0)));
            } else {
                $speedyOrderData['taking_date'] = date(time());
            }

            if (!$speedyOrderData['abroad']) {
                $speedyOrderData['cod_status'] = true;
            } else {
                $speedyOrderData['cod_status'] = false;
            }

            $speedyOrderData['user_locale'] = $this->authSession->getUser()->getInterfaceLocale();

            $speedyOrderData['offices'] = array();

            if ($speedyOrderData['city_id']) {
                $locale = $speedyOrderData['abroad'] ? 'EN' : 'BG';

                $speedyOrderData['offices'] = $this->speedyHelper->getOffices(null, $speedyOrderData['city_id'], $locale);
            }

            if ($this->speedyHelper->getError()) {
                $speedyOrderData['error'] = $this->speedyHelper->getError();
            }
        }

        return $speedyOrderData;
    }

    /**
     * @return string
     */
    public function getCreateUrl()
    {
        return $this->getUrl('extensa_speedy/order/create', array('order_id' => $this->getSpeedyOrder()->getOrderId()));
    }

    /**
     * @return string
     */
    public function getCalculateUrl()
    {
        return $this->getUrl('extensa_speedy/order/calculate', array('order_id' => $this->getSpeedyOrder()->getOrderId()));
    }

    /**
     * @return string
     */
    public function getValidateUrl()
    {
        return $this->getUrl('extensa_speedy/order/validatebilloflading', array('_secure' => true));
    }

    /**
     * ######################## TAB settings #################################
     */

    /**
     * {@inheritdoc}
     */
    public function getTabLabel()
    {
        return __('Bill Of Lading (Speedy)');
    }

    /**
     * {@inheritdoc}
     */
    public function getTabTitle()
    {
        return __('Bill Of Lading (Speedy)');
    }

    /**
     * {@inheritdoc}
     */
    public function canShowTab()
    {
        return $this->getSpeedyOrder()
            && $this->getSpeedyOrder()->getId()
            && !$this->getSpeedyOrder()->getBolId()
            && $this->_authorization->isAllowed('Extensa_Speedy::create');
    }

    /**
     * {@inheritdoc}
     */
    public function isHidden()
    {
        return false;
    }
}
