<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */
namespace Extensa\Speedy\Block\Adminhtml\Sales\Order\View\Tab;

/**
 * Extensa Speedy Create shipment tab
 */
class Printpdf extends \Magento\Sales\Block\Adminhtml\Order\AbstractOrder implements
    \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * @var \Extensa\Speedy\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     * @var \Magento\Framework\Api\FilterBuilder
     */
    protected $filterBuilder;

    /**
     * Create constructor.
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Sales\Helper\Admin $adminHelper
     * @param \Extensa\Speedy\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Framework\Api\FilterBuilder $filterBuilder
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Sales\Helper\Admin $adminHelper,
        \Extensa\Speedy\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\FilterBuilder $filterBuilder,
        array $data = []
    ) {
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterBuilder = $filterBuilder;

        parent::__construct(
            $context,
            $registry,
            $adminHelper,
            $data
        );
    }

    /**
     * Retrieve speedy order model instance
     *
     * @return \Extensa\Speedy\Model\Order
     */
    public function getSpeedyOrder()
    {
        if (!$this->_coreRegistry->registry('current_speedy_order')) {
            $order = [];

            $id = $this->_request->getParam('order_id');
            try {
                $orderIdFilter = $this->filterBuilder
                    ->setField('order_id')
                    ->setValue($id)
                    ->create();

                $searchCriteriaBuilder = $this->searchCriteriaBuilder->addFilters([$orderIdFilter])->create();

                $items = $this->orderRepository->getList($searchCriteriaBuilder)->getItems();

                $order = array_shift($items);

                $this->_coreRegistry->register('current_speedy_order', $order);
            } catch (\Exception $e) {
                $this->_logger->error('Extensa_Speedy - ' . get_class($this) . ' :: ' . $e->getMessage());
            }
        }

        return $this->_coreRegistry->registry('current_speedy_order');
    }

    /**
     * @return $this
     */
    protected function _prepareLayout()
    {
        if ($this->canShowTab()) {
            if ($this->_authorization->isAllowed('Extensa_Speedy::view_pdf')) {
                $download_button = $this->getLayout()->createBlock(
                    'Magento\Backend\Block\Widget\Button'
                )->setData(
                    [
                        'label'     => __('Print'),
                        'class'     => 'save',
                        'onclick'   => 'window.location.href=\''.$this->getUrl('extensa_speedy/order/printpdf', array('order_id' => $this->getSpeedyOrder()->getOrderId())).'\''
                    ]
                );

                $this->setChild('download_button', $download_button);
            }

            if ($this->_authorization->isAllowed('Extensa_Speedy::cancel')) {
                $cancel_button = $this->getLayout()->createBlock(
                    'Magento\Backend\Block\Widget\Button'
                )->setData(
                    [
                        'label'     => __('Cancel'),
                        'onclick'   => 'window.location.href=\''.$this->getUrl('extensa_speedy/order/cancel', array('order_id' => $this->getSpeedyOrder()->getOrderId())).'\''
                    ]
                );

                $this->setChild('cancel_button', $cancel_button);
            }
        }
        return parent::_prepareLayout();
    }

    /**
     * ######################## TAB settings #################################
     */

    /**
     * {@inheritdoc}
     */
    public function getTabLabel()
    {
        return __('Bill Of Lading');
    }

    /**
     * {@inheritdoc}
     */
    public function getTabTitle()
    {
        return __('Bill Of Lading');
    }

    /**
     * {@inheritdoc}
     */
    public function canShowTab()
    {
        return $this->getSpeedyOrder()
            && $this->getSpeedyOrder()->getId()
            && $this->getSpeedyOrder()->getBolId();
    }

    /**
     * {@inheritdoc}
     */
    public function isHidden()
    {
        return false;
    }
}
