<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Block\Adminhtml\System\Config\Form\Field;

use Magento\Backend\Block\Template\Context;
use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\Data\Form\Element\AbstractElement;

class Reloadinformation extends Field
{
    /**
     * @var string
     */
    protected $_template = 'Extensa_Speedy::system/config/reloadinformation.phtml';

    /**
     * @var string
     */
    protected $_buttonId = 'extensa_speedy_reload_information_button';

    /**
     * @param Context $context
     * @param array $data
     */
    public function __construct(
        Context $context,
        array $data = []
    ) {
        parent::__construct($context, $data);
    }

    /**
     * Remove scope label
     *
     * @param  AbstractElement $element
     * @return string
     */
    public function render(AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }

    /**
     * Return element html
     *
     * @param  AbstractElement $element
     * @return string
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * Generate collect button html
     *
     * @return string
     */
    public function getButtonHtml()
    {
        $button = $this->getLayout()->createBlock(
            'Magento\Backend\Block\Widget\Button'
        )->setData(
            [
                'id'        => $this->_buttonId,
                'label'     => __('Reload Information'),
            ]
        );

        return $button->toHtml();
    }

    /**
     * Return ajax url for reload information button
     *
     * @return string
     */
    public function getReloadInformationUrl()
    {
        return $this->getUrl('extensa_speedy/system_config/reloadinformation', array('_secure' => true));
    }

    /**
     * @return string
     */
    public function getButtonId()
    {
        return $this->_buttonId;
    }
}
