<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Block\Sales\Order\Info;

class Track extends \Magento\Framework\View\Element\Template
{
    /**
     * @var string
     */
    protected $_template = 'Extensa_Speedy::sales/order/track.phtml';

    /**
     * @var \Extensa\Speedy\Helper\Data
     */
    protected $dataHelper;

    /**
     * Core registry
     *
     * @var \Magento\Framework\Registry
     */
    protected $_coreRegistry = null;

    /**
     * @var \Extensa\Speedy\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     * @var \Magento\Framework\Api\FilterBuilder
     */
    protected $filterBuilder;

    /**
     * @var null|\Extensa\Speedy\Model\Order
     */
    protected $speedyOrder = null;

    /**
     * Track constructor.
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Framework\Api\FilterBuilder $filterBuilder
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Extensa\Speedy\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\FilterBuilder $filterBuilder,
        array $data = []
    ) {
        $this->_coreRegistry = $registry;
        $this->dataHelper = $dataHelper;
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterBuilder = $filterBuilder;

        parent::__construct($context, $data);

        $this->_isScopePrivate = true;
    }

    /**
     * Retrieve current order model instance
     *
     * @return \Magento\Sales\Model\Order
     */
    public function getOrder()
    {
        return $this->_coreRegistry->registry('current_order');
    }

    /**
     * Retrieve speedy order model instance
     *
     * @return \Extensa\Speedy\Model\Order
     */
    private function getSpeedyOrder()
    {
        if (!$this->speedyOrder) {
            $order = $this->getOrder();

            if ($order && $order->getId()) {
                $id = $order->getId();

                try {
                    $orderIdFilter = $this->filterBuilder
                        ->setField('order_id')
                        ->setValue($id)
                        ->create();

                    $searchCriteriaBuilder = $this->searchCriteriaBuilder->addFilters([$orderIdFilter])->create();

                    $items = $this->orderRepository->getList($searchCriteriaBuilder)->getItems();

                    $this->speedyOrder = array_shift($items);
                } catch (\Exception $e) {
                    $this->_logger->error('Extensa_Speedy - ' . get_class($this) . ' :: ' . $e->getMessage());
                }
            }
        }

        return $this->speedyOrder;
    }

    /**
     * @return array
     */
    public function getTracks()
    {
        $results_data = array();
        $speedyOrderInfo = $this->getSpeedyOrder();

        if ($speedyOrderInfo->getId() && $speedyOrderInfo->getBolId()) {
            $results_data['bol_id'] = $speedyOrderInfo->getBolId();
            $results_data['date_created'] = $speedyOrderInfo->getDateCreated();
            $results_data['tracks'] = \Extensa\Speedy\Helper\Speedy::TRACKING_URL . $speedyOrderInfo->getBolId();
        } else {
            $results_data['error'] = __('There is no bill of lading for this order!');
        }

        return $results_data;
    }
}
