<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Controller\Address;

class Country extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Extensa\Speedy\Helper\Speedy $speedyHelper
     */
    protected $speedyHelper;

    /**
     * @var \Magento\Checkout\Model\Session $checkoutSession
     */
    protected $checkoutSession;

    /**
     * City constructor.
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     * @param \Magento\Checkout\Model\Session $checkoutSession
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Extensa\Speedy\Helper\Speedy $speedyHelper,
        \Magento\Checkout\Model\Session $checkoutSession
    ) {
        $this->resultPageFactory = $resultPageFactory;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->speedyHelper = $speedyHelper;
        $this->checkoutSession = $checkoutSession;

        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $items = [];
        $result = $this->resultJsonFactory->create();

        try {
            if ($this->_request->isPost()) {
                $filter = array();

                if ($this->getRequest()->getParam('iso_code_2')) {
                    $filter['iso_code_2'] = $this->getRequest()->getParam('iso_code_2');
                } elseif ($this->getRequest()->getParam('name')) {
                    $filter['name'] = $this->getRequest()->getParam('name');
                } else {
                    $speedySession = $this->checkoutSession->getExtensaSpeedyCountry();

                    if (!empty($speedySession['country_id'])) {
                        $filter['iso_code_2'] = $speedySession['country_id'];
                    } else {
                        return $result->setData(['success' => false, 'message' => __('Could not find country')]);
                    }
                }

                $country = $this->speedyHelper->getCountries($filter, $this->getRequest()->getParam('locale'));

                return $result->setData(['success' => true, 'country' => $country]);
            }
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            return $result->setData(['success' => false, 'message' => $e->getMessage()]);
        } catch (\Exception $e) {
            return $result->setData(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    /**
     * @return bool
     */
    protected function _isAllowed()
    {
        return parent::_isAllowed('Extensa_Speedy::config');
    }
}
