<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Controller\Address;

class Street extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Extensa\Speedy\Helper\Speedy $speedyHelper
     */
    protected $speedyHelper;

    /**
     * Street constructor.
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Extensa\Speedy\Helper\Speedy $speedyHelper
    ) {
        $this->resultPageFactory = $resultPageFactory;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->speedyHelper = $speedyHelper;

        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $items = [];
        $result = $this->resultJsonFactory->create();

        try {
            if ($this->_request->isPost()) {
                $name = $this->getRequest()->getParam('name');
                $city_id = $this->getRequest()->getParam('city_id');
                $limit = $this->getRequest()->getParam('limit', 10);
                $locale = $this->getRequest()->getParam('locale');

                if ($city_id) {
                    $streets = $this->speedyHelper->getStreets($name, $city_id, $locale);

                    $resultData = [
                        'success'   => true,
                        'items'     => array_slice($streets, 0, $limit)
                    ];

                    if ($this->speedyHelper->getError()) {
                        $resultData = [
                            'success'   => false,
                            'message'   => $this->speedyHelper->getError()
                        ];
                    }
                } else {
                    $resultData = [
                        'success'   => false,
                        'message'   => __('Please enter a city.')
                    ];
                }

                return $result->setData($resultData);
            }
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            return $result->setData(['success' => false, 'message' => $e->getMessage()]);
        } catch (\Exception $e) {
            return $result->setData(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    /**
     * @return bool
     */
    protected function _isAllowed()
    {
        return parent::_isAllowed('Extensa_Speedy::config');
    }
}
