<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */


namespace Extensa\Speedy\Controller\Address;

class Validate extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Extensa\Speedy\Api\AddressRepositoryInterface
     */
    protected $addressRepository;

    /**
     * @var \Extensa\Speedy\Api\Data\AddressInterfaceFactory
     */
    protected $addressFactory;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $speedyHelper;

    /**
     * Validate constructor.
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Extensa\Speedy\Api\AddressRepositoryInterface $addressRepository
     * @param \Extensa\Speedy\Api\Data\AddressInterfaceFactory $addressFactory
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Extensa\Speedy\Api\AddressRepositoryInterface $addressRepository,
        \Extensa\Speedy\Api\Data\AddressInterfaceFactory $addressFactory,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Customer\Model\Session $customerSession,
        \Extensa\Speedy\Helper\Speedy $speedyHelper
    ) {
        $this->resultPageFactory    = $resultPageFactory;
        $this->resultJsonFactory    = $resultJsonFactory;
        $this->addressRepository    = $addressRepository;
        $this->addressFactory       = $addressFactory;
        $this->checkoutSession      = $checkoutSession;
        $this->customerSession      = $customerSession;
        $this->speedyHelper         = $speedyHelper;

        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $result = $this->resultJsonFactory->create();
        $resultsData = [
            'success'   => true,
            'message'   => '',
        ];

        $this->checkoutSession->unsExtensaSpeedy();

        try {
            if ($this->getRequest()->isPost()) {
                $post = $this->getRequest()->getParam('extensa_speedy');
                $post['shipping_method'] = $this->getRequest()->getParam('shipping_method');

                if (!$post['abroad']) {
                    if ($post['postcode'] && $post['city'] && $post['city_id'] &&
                        (!$post['to_office'] && (($post['quarter'] && ($post['quarter_id'] && $post['city_nomenclature'] == 'FULL' || $post['city_nomenclature'] != 'FULL') && ($post['block_no'] || $post['street_no'])) ||
                        ($post['street'] && ($post['street_id'] && $post['city_nomenclature'] == 'FULL' || $post['city_nomenclature'] != 'FULL') && ($post['block_no'] || $post['street_no'])) || $post['note']) || ($post['to_office'] && $post['office_id']))) {
                    } else {
                            if ($post['to_office']) {
                                $resultsData['success'] = false;
                                $resultsData['message'] = __('Please enter a city and select an office.');
                            } else {
                                $resultsData['success'] = false;
                                $resultsData['message'] = __('Please enter a valid address.');
                            }
                    }

                    if (isset($post['fixed_time_cb'])) {
                        if (!$post['fixed_time_hour'] || $post['fixed_time_hour'] < 10 || $post['fixed_time_hour'] > 17 ||
                            !$post['fixed_time_min'] || $post['fixed_time_min'] < '00' || $post['fixed_time_min'] > 59 ||
                            ($post['fixed_time_hour'] == 10 && $post['fixed_time_min'] < 30) ||
                            ($post['fixed_time_hour'] == 17 && $post['fixed_time_min'] > 30)) {
                                $resultsData['success'] = false;
                                $resultsData['message'] = __('Please select a valid hour.');
                        }
                    }
                } else {
                    $validAddress = $this->speedyHelper->validateAddress($post);

                    if (!$validAddress) {
                        $resultsData['success'] = false;
                        $resultsData['message'] = $this->speedyHelper->getError();
                    }
                }

                if ($resultsData['success']) {
                    $this->checkoutSession->setExtensaSpeedy($post);

                    if ($this->customerSession->getCustomer()->getId() &&
                        $this->checkoutSession->getQuote()->getShippingAddress()->getCustomerAddressId()) {
                        $post['address_id'] = $this->checkoutSession->getQuote()->getShippingAddress()->getCustomerAddressId();
                        $post['customer_id'] = $this->customerSession->getCustomer()->getId();

                        $address = $this->addressFactory->create();
                        $address->setData($post);

                        $this->addressRepository->save($address);
                    }
                }

                return $result->setData($resultsData);
            }
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            return $result->setData(['success' => false, 'message' => $e->getMessage()]);
        } catch (\Exception $e) {
            return $result->setData(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    /**
     * @return bool
     */
    protected function _isAllowed()
    {
        return parent::_isAllowed('Extensa_Speedy::config');
    }
}
