<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Controller\Adminhtml;

use Magento\Backend\App\Action;

abstract class Order extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Extensa_Speedy::orders';

    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var \Magento\Backend\Model\View\Result\ForwardFactory
     */
    protected $resultForwardFactory;

    /**
     * @var \Magento\Framework\Registry
     */
    protected $registry;

    /**
     * @var \Extensa\Speedy\Helper\Data
     */
    protected $dataHelper;

    /**
     * @var \Extensa\Speedy\Helper\Speedy
     */
    protected $speedyHelper;

    /**
     * @var \Extensa\Speedy\Api\OrderRepositoryInterface
     */
    protected $speedyOrderRepository;

    /**
     * @var \Extensa\Speedy\Api\Data\OrderInterfaceFactory
     */
    protected $speedyOrderFactory;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     * @var \Magento\Framework\Api\FilterBuilder
     */
    protected $filterBuilder;

    /**
     * @var \Magento\Sales\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Extensa\Speedy\Model\Order
     */
    protected $speedyOrder;

    /**
     * @var string
     */
    protected $refreshUrl;

    /**
     * Order constructor.
     * @param Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory
     * @param \Magento\Framework\Registry $registry
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     * @param \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository
     * @param \Extensa\Speedy\Api\Data\OrderInterfaceFactory $speedyOrderFactory
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Framework\Api\FilterBuilder $filterBuilder
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory,
        \Magento\Framework\Registry $registry,
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Extensa\Speedy\Helper\Speedy $speedyHelper,
        \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository,
        \Extensa\Speedy\Api\Data\OrderInterfaceFactory $speedyOrderFactory,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\FilterBuilder $filterBuilder,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
    ) {
        $this->resultPageFactory = $resultPageFactory;
        $this->resultForwardFactory = $resultForwardFactory;
        $this->resultRedirectFactory = $context->getResultRedirectFactory();
        $this->registry = $registry;
        $this->dataHelper = $dataHelper;
        $this->speedyHelper = $speedyHelper;
        $this->speedyOrderRepository = $speedyOrderRepository;
        $this->speedyOrderFactory = $speedyOrderFactory;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterBuilder = $filterBuilder;
        $this->orderRepository = $orderRepository;
        $this->resultJsonFactory = $resultJsonFactory;

        parent::__construct($context);
    }

    /**
     * @return bool|\Extensa\Speedy\Api\Data\OrderInterface|\Extensa\Speedy\Model\Order|mixed|null
     */
    protected function _initSpeedyOrder()
    {
        if (!$this->speedyOrder) {
            $order = [];

            $id = $this->_request->getParam('order_id');

            try {
                $orderIdFilter = $this->filterBuilder
                    ->setField('order_id')
                    ->setValue($id)
                    ->create();

                $searchCriteriaBuilder = $this->searchCriteriaBuilder->addFilters([$orderIdFilter])->create();

                $items = $this->speedyOrderRepository->getList($searchCriteriaBuilder)->getItems();

                $this->speedyOrder = array_shift($items);
            } catch (\Exception $e) {
                $this->speedyOrder = null;
                $this->getMessageManager()->addErrorMessage($e->getMessage());
            }

            if (!$this->speedyOrder
                || !$this->speedyOrder->getId()
            ) {
                $this->speedyOrder = null;
            }
        }

        return $this->speedyOrder;
    }

    public function getRefreshUrl()
    {
        if (!$this->refreshUrl) {
            $this->refreshUrl = $this->getUrl('sales/order/view', ['order_id' => $this->_request->getParam('order_id')]);
        }

        return $this->refreshUrl;
    }
}
