<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Controller\Adminhtml\Order;

use Extensa\Speedy\Controller\Adminhtml\Order;

class Calculate extends Order
{
    /**
     * @var \Extensa\Speedy\Model\TablerateRepository
     */
    protected $tablerateRepository;

    /**
     * Order constructor.
     * @param Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory
     * @param \Magento\Framework\Registry $registry
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     * @param \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository
     * @param \Extensa\Speedy\Api\Data\OrderInterfaceFactory $speedyOrderFactory
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Framework\Api\FilterBuilder $filterBuilder
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Extensa\Speedy\Model\TablerateRepository $tablerateRepository
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory,
        \Magento\Framework\Registry $registry,
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Extensa\Speedy\Helper\Speedy $speedyHelper,
        \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository,
        \Extensa\Speedy\Api\Data\OrderInterfaceFactory $speedyOrderFactory,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\FilterBuilder $filterBuilder,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Extensa\Speedy\Model\TablerateRepository $tablerateRepository
    ) {
        $this->tablerateRepository = $tablerateRepository;

        parent::__construct(
            $context,
            $resultPageFactory,
            $resultForwardFactory,
            $registry,
            $dataHelper,
            $speedyHelper,
            $speedyOrderRepository,
            $speedyOrderFactory,
            $searchCriteriaBuilder,
            $filterBuilder,
            $orderRepository,
            $resultJsonFactory
        );
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $result = $this->resultJsonFactory->create();

        if (!$this->_initSpeedyOrder()) {
            $this->messageManager->addErrorMessage(__('This bill of lading no longer exists.'));

            return $result->setData([
                'success'       => false,
                'ajaxExpired'   => true,
                'ajaxRedirect'  => $this->getRefreshUrl(),
            ]);
        }

        $speedyOrder = $this->_initSpeedyOrder();
        $shippingParams = array();
        $response = false;

        $post = $this->getRequest()->getParam($this->getRequest()->getParam('html_id'));

        if (empty($post['shipping_method'])) {
            $order = $this->orderRepository->get($speedyOrder->getOrderId());
            $shippingMethod = explode('_', $order->getShippingMethod());
            $shippingMethodId = end($shippingMethod);
        } else {
            $shippingMethodId = $post['shipping_method'];
        }

        if (isset($post['fixed_time_cb'])) {
            $post['fixed_time'] = $post['fixed_time_hour'] . $post['fixed_time_min'];
        } else {
            $post['fixed_time'] = null;
        }

        if ($shippingMethodId == 500) {
            $post['parcels_size'] = array(
                1 => array(
                    'weight' => '',
                    'width'  => '',
                    'height' => '',
                    'depth'  => '',
                )
            );
        }

        $methods = $this->speedyHelper->calculate($post);
        $services = $this->speedyHelper->getServices();
        $shippingMethods = [];

        if (!$this->speedyHelper->getError()) {
            foreach ($methods as $method) {
                if (($this->dataHelper->getStoreConfig('pricing') == 'free')
                    && ($post['total'] >= $this->dataHelper->getStoreConfig('free_shipping_total'))
                    && ($method->getServiceTypeId() == $this->dataHelper->getStoreConfig('free_method_city')
                        || $method->getServiceTypeId() == $this->dataHelper->getStoreConfig('free_method_intercity')
                        || in_array($method->getServiceTypeId(), $this->dataHelper->getStoreConfigArray('free_method_international'))
                        )
                    ) {
                    $method_total = 0;
                } elseif ($this->dataHelper->getStoreConfig('pricing') == 'fixed') {
                    $method_total = $this->dataHelper->getStoreConfig('fixed_price');
                } elseif ($this->dataHelper->getStoreConfig('pricing') == 'table_rate') {
                    $filter = $this->searchCriteriaBuilder
                        ->addFilter('service_id', (int)$method->getServiceTypeId(), 'eq')
                        ->addFilter('take_from_office', (int)$post['to_office'], 'eq')
                        ->addFilter('weight', (float)$post['weight'], 'gteq')
                        ->addFilter('order_total', (float)$post['total'], 'gteq');

                    if (!empty($post['fixed_time_cb'])) {
                        $filter = $filter->addFilter('fixed_time_delivery', (int)$post['fixed_time_cb'], 'eq')
                            ->create();
                    } else {
                        $filter = $filter->addFilter('fixed_time_delivery', 0, 'eq')
                            ->create();
                    }

                    $tableRate = $this->tablerateRepository->getFirstResult($filter);

                    if ($tableRate->getPriceWithoutVat() == null) {
                        continue;
                    } else {
                        $method_total = $tableRate->getPriceWithoutVat();
                    }
                } else {
                    $method_total = $method->getResultInfo()->getAmounts()->getTotal();

                    if ($this->dataHelper->getStoreConfig('pricing') == 'calculator_fixed') {
                        $method_total += $this->dataHelper->getStoreConfig('fixed_price');
                    }
                }

                $shippingMethods[] = array(
                    'method_id' => $method->getServiceTypeId(),
                    'total'     => $method_total,
                    'label'     => $this->dataHelper->getStoreConfig('name') . ' - ' . $services[$method->getServiceTypeId()],
                    'checked'   => ($shippingMethodId == $method->getServiceTypeId() ? true : false),
                );
            }

            if (!empty($shippingMethods)) {
                $response = [
                    'success'           => true,
                    'message'           => __('You have successfully calculated price.'),
                    'shipping_methods'  => $shippingMethods
                ];
            } elseif (empty($shippingMethods) && $this->dataHelper->getStoreConfig('pricing') == 'table_rate') {
                $response = [
                    'success'           => false,
                    'message'           => __('No shipping methods were not found')
                ];
            } else {
                $response = [
                    'success'           => false,
                    'message'           => __('There are no available Speedy methods for this address!')
                ];
            }
        } else {
            $response = [
                'success' => false,
                'message' => $this->speedyHelper->getError()
            ];
        }

        return $result->setData($response);
    }
}
