<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Controller\Adminhtml\Order;

use Extensa\Speedy\Controller\Adminhtml\Order;
use Magento\Framework\Controller\ResultFactory;

class Create extends Order
{
    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    protected $date;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Sales\Model\Order\Email\Sender\OrderCommentSender
     */
    protected $orderCommentSender;

    /**
     * @var \Extensa\Speedy\Model\TablerateRepository $tablerateRepository
     */
    protected $tablerateRepository;

    /**
     * Order constructor.
     * @param Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory
     * @param \Magento\Framework\Registry $registry
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     * @param \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository
     * @param \Extensa\Speedy\Api\Data\OrderInterfaceFactory $speedyOrderFactory
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Framework\Api\FilterBuilder $filterBuilder
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Magento\Framework\Stdlib\DateTime\DateTime $date
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Sales\Model\Order\Email\Sender\OrderCommentSender $orderCommentSender
     * @param \Extensa\Speedy\Model\TablerateRepository $tablerateRepository
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory,
        \Magento\Framework\Registry $registry,
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Extensa\Speedy\Helper\Speedy $speedyHelper,
        \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository,
        \Extensa\Speedy\Api\Data\OrderInterfaceFactory $speedyOrderFactory,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\FilterBuilder $filterBuilder,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Sales\Model\Order\Email\Sender\OrderCommentSender $orderCommentSender,
        \Extensa\Speedy\Model\TablerateRepository $tablerateRepository
    ) {
        $this->date = $date;
        $this->storeManager = $storeManager;
        $this->orderCommentSender = $orderCommentSender;
        $this->tablerateRepository = $tablerateRepository;

        parent::__construct(
            $context,
            $resultPageFactory,
            $resultForwardFactory,
            $registry,
            $dataHelper,
            $speedyHelper,
            $speedyOrderRepository,
            $speedyOrderFactory,
            $searchCriteriaBuilder,
            $filterBuilder,
            $orderRepository,
            $resultJsonFactory
        );
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $response = [];
        $resultPage = $this->resultJsonFactory->create();

        if (!$this->_initSpeedyOrder()) {
            $this->messageManager->addErrorMessage(__('This bill of lading no longer exists.'));

            return $resultPage->setData([
                'success'         => false,
                'ajaxExpired'     => true,
                'ajaxRedirect'    => $this->getRefreshUrl(),
            ]);
        }

        $speedyOrder = $this->_initSpeedyOrder();

        $post = $this->getRequest()->getParam($this->getRequest()->getParam('html_id'));
        $speedyOrderData = unserialize($speedyOrder->getDataColumn());
        $order = $this->orderRepository->get($speedyOrder->getOrderId());

        if (empty($post['shipping_method'])) {
            $shippingMethod = explode('_', $order->getShippingMethod());
            $speedyOrderData['shipping_method'] = end($shippingMethod);
        }

        if (isset($post['fixed_time_cb'])) {
            $post['fixed_time'] = $post['fixed_time_hour'] . $post['fixed_time_min'];
        } else {
            $post['fixed_time'] = null;
        }

        unset($speedyOrderData['fixed_time_cb']);
        $data = array_merge($speedyOrderData, $post);

        if ($data['shipping_method'] == 500) {
            $data['parcels_size'] = array(
                1 => array(
                    'weight' => '',
                    'width'  => '',
                    'height' => '',
                    'depth'  => '',
                )
            );
        }

        if ($this->dataHelper->getStoreConfig('pricing') == 'table_rate') {
            $filter = $this->searchCriteriaBuilder
                ->addFilter('service_id', (int)$data['shipping_method'], 'eq')
                ->addFilter('take_from_office', (int)$post['to_office'], 'eq')
                ->addFilter('weight', (float)$post['weight'], 'gteq')
                ->addFilter('order_total', (float)$post['total'], 'gteq');

            if (!empty($post['fixed_time_cb'])) {
                $filter = $filter->addFilter('fixed_time_delivery', (int)$post['fixed_time_cb'], 'eq')
                    ->create();
            } else {
                $filter = $filter->addFilter('fixed_time_delivery', 0, 'eq')
                    ->create();
            }

            $tableRate = $this->tablerateRepository->getFirstResult($filter);

            if ($tableRate->getPriceWithoutVat() != null) {
                $data['shipping_method_cost'] = $tableRate->getPriceWithoutVat();
            }
        }

        $result = $this->speedyHelper->createBillOfLading($data, $order);

        if (!$this->speedyHelper->getError() && !empty($result['bol_id'])) {
            $speedyOrder->setBolId($result['bol_id'])
                ->setDataColumn(serialize($data))
                ->setDateCreated($this->date->gmtDate());

            $this->speedyOrderRepository->save($speedyOrder);

            if (($this->dataHelper->getStoreConfig('pricing') == 'free')
                && ($post['total'] >= $this->dataHelper->getStoreConfig('free_shipping_total'))
                && ($data['shipping_method'] == $this->dataHelper->getStoreConfig('free_method_city')
                    || $data['shipping_method'] == $this->dataHelper->getStoreConfig('free_method_intercity')
                    || in_array($data['shipping_method'], $this->dataHelper->getStoreConfigArray('free_method_international'))
                    )
                ) {
                $total = 0;
            } elseif ($this->dataHelper->getStoreConfig('pricing') == 'fixed') {
                $total = $this->dataHelper->getStoreConfig('fixed_price');
            } elseif ($this->dataHelper->getStoreConfig('pricing') == 'table_rate') {
                if ($tableRate->getPriceWithoutVat() != null) {
                    $total = $tableRate->getPriceWithoutVat();
                }
            } else {
                $total = $result['total'];

                if ($this->dataHelper->getStoreConfig('pricing') == 'calculator_fixed') {
                    $total += $this->dataHelper->getStoreConfig('fixed_price');
                }
            }

            $currencyShippingRate = $this->storeManager->getStore()->getBaseCurrency()->getRate(
                $this->dataHelper->getStoreConfig('currency')
            );
            $currencyOrderRate = $this->storeManager->getStore()->getBaseCurrency()->getRate(
                $order->getOrderCurrencyCode()
            );
            $currencyBaseRate = $this->storeManager->getStore()->getBaseCurrency()->getRate(
                $order->getBaseCurrencyCode()
            );

            $shippingPrice = (float)$total * ($currencyOrderRate / $currencyShippingRate);
            $baseShippingPrice = (float)$total * ($currencyBaseRate / $currencyShippingRate);

            $order->setGrandTotal($order->getGrandTotal() - $order->getShippingAmount() + $shippingPrice);
            $order->setBaseGrandTotal($order->getBaseGrandTotal() - $order->getBaseShippingAmount() + $baseShippingPrice);

            $order->setShippingAmount($shippingPrice);
            $order->setShippingInclTax($shippingPrice);
            $order->setShippingInvoiced($shippingPrice);

            $order->setBaseShippingAmount($baseShippingPrice);
            $order->setBaseShippingInclTax($baseShippingPrice);
            $order->setBaseShippingInvoiced($baseShippingPrice);

            if (!empty($post['shipping_method']) && !empty($post['shipping_method_label_' . $post['shipping_method']])) {
                $oldShippingMethod = explode('_', $order->getShippingMethod());
                $order->setShippingMethod(str_replace(end($oldShippingMethod), $post['shipping_method'], $order->getShippingMethod()));
                $order->setShippingDescription($this->dataHelper->getStoreConfig('title') . ' - ' . $this->dataHelper->getStoreConfig('name') . ' - ' . $post['shipping_method_label_' . $post['shipping_method']]);
            }

            $receiverAddress = array();
            if (empty($post['to_office'])) {
                if (!empty($post['quarter'])) {
                    $receiverAddress[] = __('Quarter') . ': ' . $post['quarter'];
                }
                if (!empty($post['street'])) {
                    $receiverAddress[] = __('Street') . ': ' . $post['street'];
                }
                if (!empty($post['street_no'])) {
                    $receiverAddress[] = __('Number') . ': ' . $post['street_no'];
                }
                if (!empty($post['block_no'])) {
                    $receiverAddress[] = __('Apt. building') . ': ' . $post['block_no'];
                }
                if (!empty($post['entrance_no'])) {
                    $receiverAddress[] = __('Ent.') . ': ' . $post['entrance_no'];
                }
                if (!empty($post['floor_no'])) {
                    $receiverAddress[] = __('Fl.') . ': ' . $post['floor_no'];
                }
                if (!empty($post['apartment_no'])) {
                    $receiverAddress[] = __('Ap.') . ': ' . $post['apartment_no'];
                }
            }

            if (!empty($post['note'])) {
                $receiverAddress[] = __('Address Note') . ': ' . $post['note'];
            }

            $speedyCountry = $this->speedyHelper->getCountries(['country_id' => $post['country_id']]);
            $speedyCountry = next($speedyCountry);

            $order->getShippingAddress()
                ->setRegionId(null)
                ->setRegion(null)
                ->setPostcode($post['postcode'])
                ->setCity($post['city'])
                ->setStreet(implode(', ', $receiverAddress))
                ->setCountryId($speedyCountry['iso_code_2']);

            $comment = __('Shipping Method') . ': ' . $order->getShippingDescription() . "\n" . __('Shipping & Handling Tax') . ': ' . $order->getShippingAmount() . "\n" . __('Grand Total') . ': ' . $order->getGrandTotal();

            $order->setStatus($this->dataHelper->getStoreConfig('order_status'));

            $order->save();

            $this->orderCommentSender->send($order, true, $comment);

            $this->messageManager->addSuccessMessage(__('You have successfully created the shipment.'));

            $response = array(
                'success'         => true,
                'ajaxExpired'     => true,
                'ajaxRedirect'    => $this->getRefreshUrl(),
            );
        } else {
            if ($this->speedyHelper->getError()) {
                $response = array(
                    'success'         => false,
                    'ajaxExpired'     => true,
                    'ajaxRedirect'    => $this->getRefreshUrl(),
                );

                $this->messageManager->addErrorMessage($this->speedyHelper->getError());
            }
        }

        return $resultPage->setData($response);
    }
}
