<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Controller\Adminhtml\Order;

class MassCancel extends \Magento\Backend\App\Action
{
    /**
     * @var \Extensa\Speedy\Helper\Data
     */
    protected $dataHelper;

    /**
     * @var \Extensa\Speedy\Helper\Speedy
     */
    protected $speedyHelper;

    /**
     * @var \Extensa\Speedy\Api\OrderRepositoryInterface
     */
    protected $speedyOrderRepository;

    /**
     * MassCourier constructor.
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     * @param \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Extensa\Speedy\Helper\Speedy $speedyHelper,
        \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository
    ) {
        $this->dataHelper = $dataHelper;
        $this->speedyHelper = $speedyHelper;
        $this->speedyOrderRepository = $speedyOrderRepository;

        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $orderIds = $this->getRequest()->getParam('order_ids', []);
        $speedySuccessOrders = [];

        if (!empty($orderIds) && !is_array($orderIds)) {
            $orderIds = [$orderIds];
        }

        if (empty($orderIds) && !is_array($orderIds)) {
            $this->messageManager->addErrorMessage(__('Please select one or more orders.'));
        } else {
            try {
                foreach ($orderIds as $orderId) {
                    $speedyOrder = $this->speedyOrderRepository->getById($orderId);

                    if ($speedyOrder && $speedyOrder->getId() && $speedyOrder->getBolId()) {
                        $this->speedyHelper->cancelBol($speedyOrder->getBolId());

                        if (!$this->speedyHelper->getError()) {
                           $speedyOrder->setBolId('')
                               ->setDateCreated('0000-00-00 00:00:00')
                               ->setCourier(0);

                            $this->speedyOrderRepository->save($speedyOrder);

                            $speedySuccessOrders[] = $orderId;
                        } else {
                            $this->messageManager->addErrorMessage(__('Error: Could not cancel Order# %1 %2', $orderId, $this->speedyHelper->getError()));
                        }
                    }
                }
            } catch (\Exception $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            }

            if ($speedySuccessOrders) {
                $this->messageManager->addSuccessMessage(__('Successfully canseled orders with # %1', implode(', ',$speedySuccessOrders)));
            }
        }

        $this->_redirect('*/*/index');
    }
}
