<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Controller\Adminhtml\Order;

use Extensa\Speedy\Controller\Adminhtml\Order;
use Magento\Framework\App\Filesystem\DirectoryList;

class Printpdf extends Order
{
    /**
     * @var \Magento\Framework\App\Response\Http\FileFactory
     */
    protected $fileFactory;

    /**
     * Printpdf constructor.
     * @param Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory
     * @param \Magento\Framework\Registry $registry
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     * @param \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository
     * @param \Extensa\Speedy\Api\Data\OrderInterfaceFactory $speedyOrderFactory
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Framework\Api\FilterBuilder $filterBuilder
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Magento\Framework\App\Response\Http\FileFactory $fileFactory
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory,
        \Magento\Framework\Registry $registry,
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Extensa\Speedy\Helper\Speedy $speedyHelper,
        \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository,
        \Extensa\Speedy\Api\Data\OrderInterfaceFactory $speedyOrderFactory,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\FilterBuilder $filterBuilder,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Framework\App\Response\Http\FileFactory $fileFactory
    ) {
        $this->fileFactory = $fileFactory;

        parent::__construct(
            $context,
            $resultPageFactory,
            $resultForwardFactory,
            $registry,
            $dataHelper,
            $speedyHelper,
            $speedyOrderRepository,
            $speedyOrderFactory,
            $searchCriteriaBuilder,
            $filterBuilder,
            $orderRepository,
            $resultJsonFactory
        );
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $resultPage = $this->resultJsonFactory->create();

        if (!$this->_initSpeedyOrder()) {
            $this->messageManager->addErrorMessage(__('This bill of lading no longer exists.'));

            return $resultPage->setData([
                'success'   => false,
                'ajaxExpired'   => true,
                'ajaxRedirect'   => $this->getRefreshUrl(),
            ]);
        }

        $speedyOrder = $this->_initSpeedyOrder();

        $pdf = $this->speedyHelper->createPDF($speedyOrder->getBolId());

        if (!$this->speedyHelper->getError() && $pdf) {
            return $this->fileFactory->create(
                $speedyOrder->getBolId() . '.pdf',
                $pdf,
                DirectoryList::VAR_DIR,
                'application/pdf'
            );
        } elseif ($this->speedyHelper->getError()) {
            $this->messageManager->addErrorMessage($this->speedyHelper->getError());
        } else {
            $this->messageManager->addErrorMessage(__('This bill of lading no longer exists.'));
        }

        return $this->_redirect('*/order/view', array('order_id' => $speedyOrder->getOrderId()));
    }
}
