<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Controller\Adminhtml\Order;

class ValidateBillOfLading extends \Magento\Backend\App\Action
{
    /**
     * @var \Extensa\Speedy\Helper\Data
     */
    protected $dataHelper;

    /**
     * @var \Extensa\Speedy\Helper\Speedy
     */
    protected $speedyHelper;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * MassCourier constructor.
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Extensa\Speedy\Helper\Speedy $speedyHelper
    ) {
        $this->dataHelper = $dataHelper;
        $this->speedyHelper = $speedyHelper;
        $this->resultJsonFactory = $resultJsonFactory;

        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $response = array(
            'success' => true,
            'message' => ''
        );

        $resultPage = $this->resultJsonFactory->create();
        $post = $this->getRequest()->getParam($this->getRequest()->getParam('html_id'));

        if (($post['option_before_payment'] != 'no_option' ||
             $this->dataHelper->getStoreConfig('back_documents') ||
             $this->dataHelper->getStoreConfig('back_receipt')
            ) && !empty($post['to_office']) && $post['to_office'] && $post['office_id'] && $post['abroad'] && $post['city_id']) {
            $office = $this->speedyHelper->getOfficeById($post['office_id'], $post['city_id']);

            if(!empty($office) && $office->getOfficeType() == 3) { // 3 for APT office
                $response = array(
                    'success' => true,
                    'message' => __('The selected office is APT and the folowing options will not be taken into account "Test before payment", "Open before payment", "Back documents request" and "Back receipt request"')
                );
            }
        }

        // check BackDocumentsRequest and BackReceiptRequest
        if (!empty($post['shipping_method'])) {
            $service = $this->speedyHelper->getServiceById($post['shipping_method']);

            if(!empty($service)) {
                if($service->getAllowanceBackDocumentsRequest()->getValue() == 'BANNED' && $service->getAllowanceBackReceiptRequest()->getValue() == 'BANNED') {
                    $response = array(
                        'success' => true,
                        'message' => __('The folowing options will not be taken into account "Back documents request" and "Back receipt request"')
                    );
                } elseif($service->getAllowanceBackDocumentsRequest()->getValue() == 'BANNED') {
                    $response = array(
                        'success' => true,
                        'message' => __('"Back documents request" will not be taken into account')
                    );
                } elseif($service->getAllowanceBackReceiptRequest()->getValue() == 'BANNED') {
                    $response = array(
                        'success' => true,
                        'message' => __('"Back receipt request" will not be taken into account')
                    );
                }
            }
        }

        $taking_date = $this->dataHelper->getStoreConfig('taking_date') ? strtotime('+' . (int)$this->dataHelper->getStoreConfig('taking_date') . ' day', mktime(9, 0, 0)) : time();
        $first_available_date = strtotime($this->speedyHelper->getAllowedDaysForTaking(array('shipping_method_id' => $post['shipping_method'], 'taking_date' => $taking_date)));

        if (!$this->speedyHelper->getError() && $first_available_date) {
            if (date('d-m-Y', $first_available_date) != date('d-m-Y', $taking_date)) {
                $response = array(
                    'success' => true,
                    'message' => __('The first available date for taking the shipment is: %s. Would you like to create a Bill of Lading?', date('d-m-Y', $first_available_date)),
                    'data'    => array('taking_date' => date('d-m-Y', $first_available_date))
                );
            }
        } else {
            $response = array(
                'success' => false,
                'message' => $this->speedyHelper->getError()
            );
        }

        return $resultPage->setData($response);
    }
}
