<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Controller\Adminhtml\System\Config;

class Reloadinformation extends \Magento\Backend\App\Action
{
    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Extensa\Speedy\Helper\Speedy
     */
    protected $speedyHelper;

    /**
     * Reloadinformation constructor.
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Extensa\Speedy\Helper\Speedy $speedyHelper
    ) {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->speedyHelper = $speedyHelper;

        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $resultJsonFactory = $this->resultJsonFactory->create();

        if ($this->getRequest()->isPost()) {
            if ($this->speedyHelper->getResultLogin() && !$this->speedyHelper->getError()) {
                $services = $this->speedyHelper->getServices();
                $clients = $this->speedyHelper->getListContractClients();
                $offices = $this->speedyHelper->getOffices();

                if ($services && $clients) {
                    $response['error'] = false;

                    foreach ($services as $service_id => $service) {
                        $response['services'][$service_id] = $service . __(' (Service ID: %1)', $service_id);
                    }

                    foreach ($clients as $client_id => $client) {
                        $response['clients'][$client_id] = __('ID: %1, Name: %2, Address: %3', $client_id, $client['name'], $client['address']);
                    }

                    foreach ($offices as $offices_id => $office) {
                        $response['offices'][$offices_id] = $office['label'];
                    }

                    return $resultJsonFactory->setData($response);
                } else {
                    return $resultJsonFactory->setData(['error' => true, 'message' => __('Error occured: Could not get services or clients!')]);
                }
            } else {
                return $resultJsonFactory->setData(['error' => true, 'message' => __('Error occured: Could not authenticate to Speedy!')]);
            }
        }
    }

    /**
     * @return bool
     */
    protected function _isAllowed()
    {
        return parent::_isAllowed('Extensa_Speedy::config');
    }
}
