<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\Module\ModuleListInterface;
use \Magento\Framework\Url;

class Data extends AbstractHelper
{
    const XML_PATH = 'carriers/extensaspeedy/';
    const MODULE_NAME = 'Extensa_Speedy';

    const PAYER_SENDER = 0;
    const PAYER_RECEIVER = 1;

    const BULGARIA = 'BG';

    protected $parcelSize = array(
       1 => array('size' => 'XS', 'height' => 50, 'width' => 35, 'depth' => 3.5),
       2 => array('size' => 'S' , 'height' => 60, 'width' => 35, 'depth' => 11),
       3 => array('size' => 'M' , 'height' => 60, 'width' => 35, 'depth' => 19),
       4 => array('size' => 'L' , 'height' => 60, 'width' => 35, 'depth' => 37),
       5 => array('size' => 'XL', 'height' => 60, 'width' => 60, 'depth' => 60)
    );

    protected $tableRateColumn = array(
        'ServiceID'          => 'service_id',
        'TakeFromOffice'     => 'take_from_office',
        'Weight'             => 'weight',
        'OrderTotal'         => 'order_total',
        'PriceWithoutVAT'    => 'price_without_vat',
        'FixedTimeDelivery'  => 'fixed_time_delivery'
    );

    /**
     * @var \Magento\Framework\App\Helper\Context
     */
    protected $context;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Framework\Module\ModuleListInterface
     */
    protected $moduleList;

    /**
     * @var \Magento\Framework\Url
     */
    protected $urlHelper;

    /**
     * Data constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Magento\Framework\Url $urlHelper
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Url $urlHelper
    ) {
        $this->context          = $context;
        $this->storeManager     = $storeManager;
        $this->moduleList       = $moduleList;
        $this->urlHelper        = $urlHelper;

        parent::__construct($context);
    }

    /**
     * @param string $code
     * @param integer $storeId
     * @return mixed
     */
    public function getStoreConfig($code, $storeId = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH . $code, ScopeInterface::SCOPE_STORE, $storeId
        );
    }

    /**
     * @param string $code
     * @param integer $storeId
     * @return bool
     */
    public function getStoreConfigFlag($code, $storeId = null)
    {
        return $this->scopeConfig->isSetFlag(
            self::XML_PATH . $code, ScopeInterface::SCOPE_STORE, $storeId
        );
    }

    /**
     * @param string $code
     * @param integer $storeId
     * @return mixed
     */
    public function getStoreConfigArray($code, $storeId = null)
    {
        return explode(',', $this->scopeConfig->getValue(
            self::XML_PATH . $code, ScopeInterface::SCOPE_STORE, $storeId
        ));
    }

    /**
     * @return string
     */
    public function getVersion()
    {
        return $this->moduleList
            ->getOne(self::MODULE_NAME)['setup_version'];
    }

    /**
     * @return array
     */
    public function getOptionsBeforePayment()
    {
        return array(
            'no_option' => __('no option'),
            'test'      => __('Test before payment'),
            'open'      => __('Open before payment'),
        );
    }

    /**
     * @return array
     */
    public function getPayers()
    {
        return array(
            self::PAYER_SENDER   => __('Sender'),
            self::PAYER_RECEIVER => __('Receiver'),
        );
    }

    /**
     * @return array
     */
    public function getPricingTypes()
    {
        return array(
            'calculator'         => __('Speedy calculator'),
            'calculator_fixed'   => __('Speedy calculator + handling fee'),
            'fixed'              => __('Fixed price'),
            'free'               => __('Free shipping'),
            'table_rate'         => __('Price from file')
        );
    }

    /**
     * @return array
     */
    public function getParcelSizeList()
    {
        return $this->parcelSize;
    }

    /**
     * @return string
     */
    public function getParcelSize($key)
    {
        if ($key && !empty($this->parcelSize[$key])) {
            return $this->parcelSize[$key]['size'];
        } else {
            return '';
        }
    }

    /**
     * @return string
     */
    public function getNextParcelSize($size)
    {
        if ($size) {
            $next = false;

            foreach ($this->parcelSize as $parcelSize) {
                if ($next) {
                    return $parcelSize['size'];
                } elseif ($parcelSize['size'] == $size) {
                    $next = true;
                }
            }
        }

        return '';
    }

    /**
     * @return string
     */
    public function compareParcelSizes($size1, $size2)
    {
        $sizes = array();

        foreach ($this->parcelSize as $key => $value) {
            $sizes[] = $value['size'];
        }

        if (!in_array($size1, $sizes) || !in_array($size2, $sizes)) {
            return '';
        }

        if (array_search($size1, $sizes) < array_search($size2, $sizes)) {
            return $size2;
        } else {
            return $size1;
        }
    }

    /**
     * @return string
     */
    public function getTableRateColumn($csvColumn)
    {
        if (!empty($this->tableRateColumn[$csvColumn])) {
            return $this->tableRateColumn[$csvColumn];
        } else {
            return '';
        }
    }

    /**
     * @return string
     */
    public function getAutocompleteCityUrl()
    {
        return $this->urlHelper->getUrl('extensa_speedy/address/city', array('_secure' => true));
    }

    /**
     * @return string
     */
    public function getAutocompleteQuarterUrl()
    {
        return $this->urlHelper->getUrl('extensa_speedy/address/quarter', array('_secure' => true));
    }

    /**
     * @return string
     */
    public function getAutocompleteStreetUrl()
    {
        return $this->urlHelper->getUrl('extensa_speedy/address/street', array('_secure' => true));
    }

    /**
     * @return string
     */
    public function getAutocompleteStateUrl()
    {
        return $this->urlHelper->getUrl('extensa_speedy/address/state', array('_secure' => true));
    }

    /**
     * @return string
     */
    public function getCountryUrl()
    {
        return $this->urlHelper->getUrl('extensa_speedy/address/country', array('_secure' => true));
    }

    /**
     * @return string
     */
    public function getOfficeUrl()
    {
        return $this->urlHelper->getUrl('extensa_speedy/address/office', array('_secure' => true));
    }

    /**
     * @return string
     */
    public function getValidateUrl()
    {
        return $this->urlHelper->getUrl('extensa_speedy/address/validate', array('_secure' => true));
    }
}
