<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Helper;

use Magento\Framework\App\Helper\AbstractHelper;

class Speedy extends AbstractHelper
{
    const SERVER_ADDRESS = 'https://www.speedy.bg/eps/main01.wsdl';

    const TRACKING_URL = 'http://www.speedy.bg/begin.php?shipmentNumber=';

    const CLIENT_SYSTEM_ID = 1804019300;

    /**
     * @var \Magento\Framework\App\Helper\Context
     */
    protected $context;

    /**
     * @var \Extensa\Speedy\Helper\Data
     */
    protected $dataHelper;

    /**
     * @var \EPSFacade
     */
    protected $ePSFacade;

    /**
     * @var \ResultLogin
     */
    protected $resultLogin;

    /**
     * @var string
     */
    protected $error;

    /**
     * @var string
     */
    protected $storeLocale;

    /**
     * @var \Magento\Directory\Model\CurrencyFactory
     */
    protected $currencyFactory;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * Speedy constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Magento\Store\Api\Data\StoreInterface $store
     * @param \Magento\Directory\Model\CurrencyFactory $currencyFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Magento\Framework\Locale\Resolver $resolver,
        \Magento\Directory\Model\CurrencyFactory $currencyFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    ) {
        $this->context = $context;
        $this->dataHelper = $dataHelper;
        $this->storeLocale = $resolver->getLocale();
        $this->currencyFactory = $currencyFactory;
        $this->storeManager = $storeManager;

        parent::__construct($context);

        $this->initConnection();
    }

    private function initConnection()
    {
        try {
             if ($this->_getRequest()->getPost('server_address')) {
                $server_address = $this->_getRequest()->getPost('server_address');
            } elseif ($this->dataHelper->getStoreConfig('server_address')) {
                $server_address = $this->dataHelper->getStoreConfig('server_address');
            } else {
                $server_address = self::SERVER_ADDRESS;
            }

            if ($this->_getRequest()->getPost('username')) {
                $username = $this->_getRequest()->getPost('username');
            } else {
                $username = $this->dataHelper->getStoreConfig('username');
            }

            if ($this->_getRequest()->getPost('password')) {
                $password = $this->_getRequest()->getPost('password');
            } else {
                $password = $this->dataHelper->getStoreConfig('password');
            }

            $ePSSOAPInterfaceImpl = new \EPSSOAPInterfaceImpl($server_address);
            $this->ePSFacade = new \EPSFacade($ePSSOAPInterfaceImpl, $username, $password);

            $this->resultLogin = $this->ePSFacade->getResultLogin();
        } catch (\Exception $e) {
            $this->error = $e->getMessage();
            $this->_logger->debug('Speedy :: initConnection :: ' . $e->getMessage());
        }
    }

    public function getListContractClients()
    {
        $clients = array();

        if (isset($this->resultLogin)) {
            try {
                $listClients = $this->ePSFacade->listContractClients();

                foreach ($listClients as $client) {
                    $address = $client->getAddress();
                    $address_string = $address->getSiteType()
                        . $address->getSiteName() . ', '
                        . $address->getRegionName() . ', '
                        . $address->getStreetType()
                        . $address->getStreetName() . ' '
                        . $address->getPostCode();

                    $name = array();

                    if (!empty($client->getPartnerName())) {
                        $name[] = $client->getPartnerName();
                    }

                    if (!empty($client->getObjectName())) {
                        $name[] = $client->getObjectName();
                    }

                    $clients[$client->getClientId()] = array(
                        'clientId'   => $client->getClientId(),
                        'name'       => implode(', ', $name),
                        'address'    => $address_string
                    );
                }
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: getServices :: ' . $e->getMessage());
            }
        }

        return $clients;
    }

    public function getServices()
    {
        $services = array();

        if ($this->storeLocale != 'en_EN') {
            $lang = 'BG';
        } else {
            $lang = 'EN';
        }

        if (isset($this->resultLogin)) {
            try {
                $listServices = $this->ePSFacade->listServices(time(), $lang);

                if ($listServices) {
                    foreach ($listServices as $service) {
                        if ($service->getTypeId() == 26 || $service->getTypeId() == 36) {
                            continue;
                        }

                        // Remove pallet services
                        if ($service->getCargoType() == 2) {
                            continue;
                        }

                        $services[$service->getTypeId()] = $service->getName();
                    }
                }
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: getServices :: ' . $e->getMessage());
            }
        }

        return $services;
    }

    public function getServiceById($service_id)
    {
        $services = array();

        if ($this->storeLocale != 'en_EN') {
            $lang = 'BG';
        } else {
            $lang = 'EN';
        }

        try {
            $servises = $this->ePSFacade->listServices(time(), $lang);

            foreach ($servises as $servise) {
                if ($servise->getTypeId() == $service_id) {
                    return $servise;
                }
            }
        } catch (\Exception $e) {
            $this->error = $e->getMessage();
            $this->_logger->debug('Speedy :: getServiceById :: ' . $e->getMessage());
        }
    }

    public function getReturnPackageServiceTypeId($picking)
    {
        $services = array();
        $returnVoucherServiceTypeId = null;

        $sender = $picking->getSender();
        $receiver = $picking->getReceiver();

        try {
            if ($this->dataHelper->getStoreConfig('from_office') && $this->dataHelper->getStoreConfig('office_id')) {
                $senderSiteId = null;
                $senderOfficeId = $this->dataHelper->getStoreConfig('office_id');
            } else {
                $senderData = $this->ePSFacade->getClientById($sender->getClientId());
                $senderSiteId = $senderData->getAddress()->getSiteId();
                $senderOfficeId = null;
            }

            if ($receiver->getAddress()) {
                $receiverSiteId = $receiver->getAddress()->getSiteId();
                $receiverOfficeId = null;
            } else {
                $receiverSiteId = null;
                $receiverOfficeId = $picking->getOfficeToBeCalledId();
            }

            // Reverse sender and receiver data
            $listServices = $this->ePSFacade->listServicesForSites(time(), $receiverSiteId, $senderSiteId, null, null, null, null, null, null, null, $receiverOfficeId, $senderOfficeId);

            foreach($listServices as $listService) {
                $services[] = $listService->getTypeId();
            }

            if (in_array($this->dataHelper->getStoreConfig('return_package_city_service_id'), $services)) {
                $returnVoucherServiceTypeId = $this->dataHelper->getStoreConfig('return_package_city_service_id');
            } elseif (in_array($this->dataHelper->getStoreConfig('return_package_intercity_service_id'), $services)) {
                $returnVoucherServiceTypeId = $this->dataHelper->getStoreConfig('return_package_intercity_service_id');
            }

        } catch (\Exception $e) {
            $this->error = $e->getMessage();
            $this->_logger->debug('Speedy :: getReturnPackageServiceTypeId :: ' . $e->getMessage());
        }

        return $returnVoucherServiceTypeId;
    }

    public function getReturnVoucherServiceTypeId($picking)
    {
        $services = array();
        $returnVoucherServiceTypeId = null;

        $sender = $picking->getSender();
        $receiver = $picking->getReceiver();

        try {
            if ($this->dataHelper->getStoreConfig('from_office') && $this->dataHelper->getStoreConfig('office_id')) {
                $senderSiteId = null;
                $senderOfficeId = $this->dataHelper->getStoreConfig('office_id');
            } else {
                $senderData = $this->ePSFacade->getClientById($sender->getClientId());
                $senderSiteId = $senderData->getAddress()->getSiteId();
                $senderOfficeId = null;
            }

            if ($receiver->getAddress()) {
                $receiverSiteId = $receiver->getAddress()->getSiteId();
                $receiverOfficeId = null;
            } else {
                $receiverSiteId = null;
                $receiverOfficeId = $picking->getOfficeToBeCalledId();
            }

            // Reverse sender and receiver data
            $listServices = $this->ePSFacade->listServicesForSites(time(), $receiverSiteId, $senderSiteId, null, null, null, null, null, null, null, $receiverOfficeId, $senderOfficeId);

            foreach($listServices as $listService) {
                $services[] = $listService->getTypeId();
            }

            if (in_array($this->dataHelper->getStoreConfig('return_voucher_city_service_id'), $services)) {
                $returnVoucherServiceTypeId = $this->dataHelper->getStoreConfig('return_voucher_city_service_id');
            } elseif (in_array($this->dataHelper->getStoreConfig('return_voucher_intercity_service_id'), $services)) {
                $returnVoucherServiceTypeId = $this->dataHelper->getStoreConfig('return_voucher_intercity_service_id');
            }

        } catch (\Exception $e) {
            $this->error = $e->getMessage();
            $this->_logger->debug('Speedy :: getReturnVoucherServiceTypeId :: ' . $e->getMessage());
        }

        return $returnVoucherServiceTypeId;
    }

    public function getOffices($name = null, $city_id = null, $locale = 'BG')
    {
        $offices = array();

        if (isset($this->resultLogin)) {
            try {
                $listOffices = $this->ePSFacade->listOfficesEx($name, $city_id, $locale);

                if ($listOffices) {
                    foreach ($listOffices as $office) {
                        $offices[$office->getId()] = array(
                            'id'    => $office->getId(),
                            'label' => $office->getId() . ' ' . $office->getName() . ', ' . $office->getAddress()->getFullAddressString(),
                            'value' => $office->getName()
                        );
                    }
                }
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: getOffices :: ' . $e->getMessage());
            }
        }

        return $offices;
    }

    public function getOfficeById($office_id, $city_id = null)
    {
        $result = '';
        if ($this->storeLocale != 'en_EN') {
            $lang = 'BG';
        } else {
            $lang = 'EN';
        }

        if (isset($this->resultLogin)) {
            try {
                $listOffices = $this->ePSFacade->listOfficesEx(null, $city_id, $lang);

                if ($listOffices) {
                    foreach ($listOffices as $office) {
                        if ($office->getId() == $office_id) {
                            $result = $office;
                            break;
                        }
                    }
                }
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: getOfficeById :: ' . $e->getMessage());
            }
        }

        return $result;
    }

    public function getCities($name = null, $postcode = null, $country_id = null, $locale)
    {
        $listSites = array();

        if (isset($this->resultLogin)) {
            try {
                $paramFilterSite = new \ParamFilterSite();

                if ($postcode) {
                    $paramFilterSite->setName($name);
                    $paramFilterSite->setPostCode($postcode);
                } else {
                    $paramFilterSite->setSearchString($name);
                }

                if ($country_id) {
                    $paramFilterSite->setCountryId($country_id);
                }

                $listSitesEx = $this->ePSFacade->listSitesEx($paramFilterSite, $locale);

                foreach ($listSitesEx as $result) {
                    if ($result->isExactMatch()) {
                        $listSites[] = $result->getSite();
                    }
                }
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: getCities :: ' . $e->getMessage());
            }
        }

        return $listSites;
    }

    public function getCountries($filter = '', $locale = 'BG')
    {
        $countries = array();
        $nomenclature = array(
            0 => 'NO',
            1 => 'FULL',
            2 => 'PARTIAL',
        );

        $paramFilterCountry = new \ParamFilterCountry();

        if (!is_array($filter)) {
            $paramFilterCountry->setName($filter);
        } else {
            if (isset($filter['country_id'])) {
                $paramFilterCountry->setCountryId($filter['country_id']);
            }

            if (isset($filter['name'])) {
                $paramFilterCountry->setName($filter['name']);
            }

            if (isset($filter['iso_code_2'])) {
                $paramFilterCountry->setIsoAlpha2($filter['iso_code_2']);
            }
        }

        if (isset($this->resultLogin)) {
            try {
                $listCountries = $this->ePSFacade->listCountriesEx($paramFilterCountry, strtoupper($locale));

                if ($listCountries) {
                    foreach ($listCountries as $country) {
                        $countries[$country->getIsoAlpha2()] = array(
                            'id'                   => $country->getCountryId(),
                            'name'                 => $country->getName(),
                            'iso_code_2'           => $country->getIsoAlpha2(),
                            'iso_code_3'           => $country->getIsoAlpha3(),
                            'nomenclature'         => $nomenclature[$country->getSiteNomen()],
                            'required_state'       => (int)$country->isRequireState(),
                            'required_postcode'    => (int)$country->isRequirePostCode(),
                            'active_currency_code' => $country->getActiveCurrencyCode(),
                        );
                    }
                }
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: getCountries :: ' . $e->getMessage());
            }
        }

        return $countries;
    }

    public function getQuarters($name = null, $city_id = null, $locale)
    {
        $quarters = array();

        if (isset($this->resultLogin)) {
            try {
                $listQuarters = $this->ePSFacade->listQuarters($name, $city_id, $locale);

                if ($listQuarters) {
                    foreach ($listQuarters as $quarter) {
                        $quarters[$quarter->getId()] = array(
                            'id'    => $quarter->getId(),
                            'label' => ($quarter->getType() ? $quarter->getType() . ' ' : '') . $quarter->getName(),
                            'value' => ($quarter->getType() ? $quarter->getType() . ' ' : '') . $quarter->getName()
                        );
                    }
                }
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: getQuarters :: ' . $e->getMessage());
            }
        }

        return $quarters;
    }

    public function getStreets($name = null, $city_id = null, $locale)
    {
        $streets = array();

        if (isset($this->resultLogin)) {
            try {
                $listStreets = $this->ePSFacade->listStreets($name, $city_id, $locale);

                if ($listStreets) {
                    foreach ($listStreets as $street) {
                        $streets[$street->getId()] = array(
                            'id'    => $street->getId(),
                            'label' => ($street->getType() ? $street->getType() . ' ' : '') . $street->getName(),
                            'value' => ($street->getType() ? $street->getType() . ' ' : '') . $street->getName()
                        );
                    }
                }
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: getStreets :: ' . $e->getMessage());
            }
        }

        return $streets;
    }

    public function getStates($country_id, $name = null)
    {
        $states = array();

        if (isset($this->resultLogin)) {
            try {
                $listStates = $this->ePSFacade->listStates($country_id, $name);

                if ($listStates) {
                    foreach ($listStates as $state) {
                        $states[$state->getStateId()] = array(
                            'id'               => $state->getStateId(),
                            'name'             => $state->getName(),
                            'code'             => $state->getStateAlpha(),
                            'country_id'       => $state->getCountryId(),
                        );
                    }
                }
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: getStates :: ' . $e->getMessage());
            }
        }

        return $states;
    }

    public function calculate($data)
    {
        $resultCalculation = array();

        if (isset($this->resultLogin)) {
            try {
                $paramCalculation = new \ParamCalculation();
                $paramCalculation->setSenderId($data['client_id']);
                $paramCalculation->setBroughtToOffice($this->dataHelper->getStoreConfig('from_office') && $this->dataHelper->getStoreConfig('office_id'));
                $paramCalculation->setToBeCalled(!$data['abroad'] && $data['to_office'] && $data['office_id']);
                $paramCalculation->setWeightDeclared($data['weight']);
                $paramCalculation->setDocuments($this->dataHelper->getStoreConfig('documents'));
                $paramCalculation->setPalletized(false);

                $parcel_sizes = array();
                $parcel_weight = 0;
                $seqNo = 1;

                if (!empty($data['parcels_size'])) {
                    foreach ($data['parcels_size'] as $parcels_size) {
                        if (!empty($parcels_size)) {
                            $paramParcelInfo = new \ParamParcelInfo();
                            $paramParcelInfo->setSeqNo($seqNo++);
                            $paramParcelInfo->setParcelId(-1);

                            if (!empty($parcels_size['depth']) || !empty($parcels_size['height']) || !empty($parcels_size['width'])) {
                                $size = new \Size();

                                if ($parcels_size['depth']) {
                                    $size->setDepth($parcels_size['depth']);
                                }

                                if ($parcels_size['height']) {
                                    $size->setHeight($parcels_size['height']);
                                }

                                if ($parcels_size['width']) {
                                    $size->setWidth($parcels_size['width']);
                                }

                                $paramParcelInfo->setSize($size);
                            } elseif(!empty($data['parcel_size'])) {
                                $paramParcelInfo->setPredefinedSize($data['parcel_size']);
                            }

                            if (!empty($parcels_size['weight'])) {
                                $paramParcelInfo->setWeight($parcels_size['weight']);

                                $parcel_weight += $parcels_size['weight'];
                            }

                            $parcel_sizes[] = $paramParcelInfo;
                        }
                    }

                    if (count($parcel_sizes) == 1 && empty($parcel_sizes[0]->getWeight())) {
                        $parcel_sizes[0]->setWeight($data['weight']);
                    }

                    if ($parcel_weight) {
                        $paramCalculation->setWeightDeclared($parcel_weight);
                    }

                    $paramCalculation->setParcels($parcel_sizes);
                }

                $paramCalculation->setParcelsCount(count($parcel_sizes));

                if (!empty($data['fixed_time'])) {
                    $paramCalculation->setFixedTimeDelivery($data['fixed_time']);
                } else {
                    $paramCalculation->setFixedTimeDelivery(null);
                }

                if ($this->dataHelper->getStoreConfig('pricing') == 'free' || $this->dataHelper->getStoreConfig('pricing') == 'fixed' || $this->dataHelper->getStoreConfig('pricing') == 'table_rate') {
                    $payerType = \ParamCalculation::PAYER_TYPE_SENDER;
                } elseif (isset($data['payer_type'])) {
                    $payerType = $data['payer_type'];
                } elseif ($this->dataHelper->getStoreConfig('pricing') == 'calculator' || $this->dataHelper->getStoreConfig('pricing') == 'calculator_fixed') {
                    if (!empty($data['abroad'])) {
                        $payerType = \ParamCalculation::PAYER_TYPE_SENDER;
                    } else {
                        $payerType = \ParamCalculation::PAYER_TYPE_RECEIVER;
                    }
                } else {
                    $payerType = \ParamCalculation::PAYER_TYPE_RECEIVER;
                }

                if ($data['abroad'] && $data['active_currency_code']) {
                    $rateToBase = $this->currencyFactory
                        ->create()
                        ->load($order->getOrderCurrencyCode())
                        ->getAnyRate($data['active_currency_code']);

                    $rateFromBase = $this->storeManager
                        ->getStore()
                        ->getBaseCurrency()
                        ->getRate($this->dataHelper->getStoreConfig('currency'));

                    if ($data['insurance']) {
                        $data['insurance_total'] = $data['insurance_total'] * ($rateToBase / $rateFromBase);
                    }

                    $data['total'] = $data['total'] * ($rateToBase / $rateFromBase);
                }

                if (isset($data['loading'])) {
                    if ($data['insurance']) {
                        if ($data['fragile']) {
                            $paramCalculation->setFragile(true);
                        } else {
                            $paramCalculation->setFragile(false);
                        }

                        $paramCalculation->setAmountInsuranceBase($data['insurance_total']);
                        $paramCalculation->setPayerTypeInsurance($payerType);
                    } else {
                        $paramCalculation->setFragile(false);
                    }
                } elseif ($this->dataHelper->getStoreConfig('insurance')) {
                    if ($this->dataHelper->getStoreConfig('fragile')) {
                        $paramCalculation->setFragile(true);
                    } else {
                        $paramCalculation->setFragile(false);
                    }

                    $paramCalculation->setAmountInsuranceBase($data['insurance_total']);
                    $paramCalculation->setPayerTypeInsurance($payerType);
                } else {
                    $paramCalculation->setFragile(false);
                }

                if (!( !$data['abroad'] && $data['to_office'] && $data['office_id'])) {
                    $paramCalculation->setReceiverSiteId($data['city_id']);
                }

                $paramCalculation->setPayerType($payerType);

                if (isset($data['cod']) && $data['cod'] && (!$this->dataHelper->getStoreConfig('money_transfer') || ($this->dataHelper->getStoreConfig('money_transfer') && $data['abroad']))) {
                    $paramCalculation->setAmountCodBase($data['total']);
                } else {
                    $paramCalculation->setAmountCodBase(0);
                }

                $paramCalculation->setTakingDate($data['taking_date']);
                $paramCalculation->setAutoAdjustTakingDate(true);

                if ($this->dataHelper->getStoreConfig('from_office') && $this->dataHelper->getStoreConfig('office_id')) {
                    $paramCalculation->setWillBringToOfficeId($this->dataHelper->getStoreConfig('office_id'));
                }

                if (!$data['abroad'] && $data['to_office'] && $data['office_id']) {
                    $paramCalculation->setOfficeToBeCalledId($data['office_id']);
                } else { 
                    $paramCalculation->setOfficeToBeCalledId(null);
                }

                if (isset($data['country_id']) && $data['abroad']) {
                    $paramCalculation->setReceiverCountryId($data['country_id']);
                    $paramCalculation->setReceiverPostCode($data['postcode']);
                }

                if (!empty($data['abroad']) && !empty($data['cod']) && ($this->dataHelper->getStoreConfig('pricing') == 'calculator' || $this->dataHelper->getStoreConfig('pricing') == 'calculator_fixed')) {
                    $paramCalculation->setIncludeShippingPriceInCod(true);
                }

                $resultCalculation = $this->ePSFacade->calculateMultipleServices($paramCalculation, $this->dataHelper->getStoreConfigArray('allowed_methods'));

                foreach ($resultCalculation as $key => $service) {
                    if ($service->getErrorDescription()) {
                        unset($resultCalculation[$key]);
                    }
                }

                $resultCalculation = array_values($resultCalculation);
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: calculate :: ' . $e->getMessage());
            }
        }

        return $resultCalculation;
    }

    public function validateAddress($address)
    {
        $paramAddress = new \ParamAddress();

        $paramAddress->setSiteId(trim($address['city_id']));
        if (!isset($address['city_id']) || !$address['city_id']) {
            $paramAddress->setSiteName(trim($address['city']));
        }

        $paramAddress->setPostCode(trim($address['postcode']));
        $paramAddress->setFrnAddressLine1(trim($address['address_1']));
        $paramAddress->setFrnAddressLine2(trim($address['address_2']));
        $paramAddress->setCountryId(trim($address['country_id']));
        $paramAddress->setStateId(trim($address['state_id']));

        try {
            $valid = $this->ePSFacade->validateAddress($paramAddress, 2);
        } catch (\Exception $e) {
            $valid = false;

            $this->error = $e->getMessage();
        }

        return $valid;
    }

    public function createBillOfLading($data, $order)
    {
        $bol = array();

        if (isset($this->resultLogin)) {
            try {
                $sender = new \ParamClientData();
                $sender->setClientId($data['client_id']);

                if ($this->dataHelper->getStoreConfig('contact_phone')) {
                    $senderPhone = new \ParamPhoneNumber();
                    $senderPhone->setNumber($this->dataHelper->getStoreConfig('contact_phone'));
                    $sender->setPhones(array(0 => $senderPhone));
                }

                if ($this->dataHelper->getStoreConfig('name')) {
                    $sender->setContactName($this->dataHelper->getStoreConfig('name'));
                }

                $receiverAddress = new \ParamAddress();
                if ($data['city_id']) {
                    $receiverAddress->setSiteId($data['city_id']);
                } else {
                    $receiverAddress->setSiteName($data['city']);
                }

                if (isset($data['quarter']) && $data['quarter']) {
                    $receiverAddress->setQuarterName($data['quarter']);
                }

                if (isset($data['quarter_id']) && $data['quarter_id']) {
                    $receiverAddress->setQuarterId($data['quarter_id']);
                }

                if (isset($data['street']) && $data['street']) {
                    $receiverAddress->setStreetName($data['street']);
                }

                if (isset($data['street_id']) && $data['street_id']) {
                    $receiverAddress->setStreetId($data['street_id']);
                }

                if (isset($data['street_no']) && $data['street_no']) {
                    $receiverAddress->setStreetNo($data['street_no']);
                }

                if (isset($data['block_no']) && $data['block_no']) {
                    $receiverAddress->setBlockNo($data['block_no']);
                }

                if (isset($data['entrance_no']) && $data['entrance_no']) {
                    $receiverAddress->setEntranceNo($data['entrance_no']);
                }

                if (isset($data['floor_no']) && $data['floor_no']) {
                    $receiverAddress->setFloorNo($data['floor_no']);
                }

                if (isset($data['apartment_no']) && $data['apartment_no']) {
                    $receiverAddress->setApartmentNo($data['apartment_no']);
                }

                if (isset($data['note']) && $data['note']) {
                    $receiverAddress->setAddressNote($data['note']);
                }

                if (isset($data['state_id']) && $data['state_id']) {
                    $receiverAddress->setStateId($data['state_id']);
                }

                if (isset($data['country_id'])) {
                    $receiverAddress->setPostCode($data['postcode']);
                    $receiverAddress->setFrnAddressLine1($data['address_1']);
                    $receiverAddress->setFrnAddressLine2($data['address_2']);
                    $receiverAddress->setCountryId($data['country_id']);
                    $receiverAddress->setStateId($data['state_id']);
                }

                $receiver = new \ParamClientData();
                $receiverPhone = new \ParamPhoneNumber();
                $receiverPhone->setNumber($order->getShippingAddress()->getTelephone());
                $receiver->setPhones(array(0 => $receiverPhone));

                if ($order->getShippingAddress()->getCompany()) {
                    $receiver->setContactName($order->getShippingAddress()->getFirstname() . ' ' . $order->getShippingAddress()->getLastname());
                    $receiver->setPartnerName($order->getShippingAddress()->getCompany());
                } else {
                    $receiver->setPartnerName($order->getShippingAddress()->getFirstname() . ' ' . $order->getShippingAddress()->getLastname());
                }

                $receiver->setEmail($order->getShippingAddress()->getEmail());

                $picking = new \ParamPicking();
                $picking->setClientSystemId(self::CLIENT_SYSTEM_ID);
                $picking->setRef1($order->getOrderId());
                $picking->setWeightDeclared($data['weight']);

                if (!empty($data['convertion_to_win1251'])) {
                    $picking->setAutomaticConvertionToWin1251(true);
                }

                $parcel_sizes = array();
                $parcel_weight = 0;
                $seqNo = 1;

                if (!empty($data['parcels_size'])) {
                    foreach ($data['parcels_size'] as $parcels_size) {
                        if ($parcels_size) {
                            $paramParcelInfo = new \ParamParcelInfo();
                            $paramParcelInfo->setSeqNo($seqNo++);
                            $paramParcelInfo->setParcelId(-1);

                            if (!empty($parcels_size['depth']) || !empty($parcels_size['height']) || !empty($parcels_size['width'])) {
                                $size = new \Size();

                                if ($parcels_size['depth']) {
                                    $size->setDepth($parcels_size['depth']);
                                }

                                if ($parcels_size['height']) {
                                    $size->setHeight($parcels_size['height']);
                                }

                                if ($parcels_size['width']) {
                                    $size->setWidth($parcels_size['width']);
                                }

                                $paramParcelInfo->setSize($size);
                            } elseif(!empty($data['parcel_size'])) {
                                $paramParcelInfo->setPredefinedSize($data['parcel_size']);
                            }

                            if (!empty($parcels_size['weight'])) {
                                $paramParcelInfo->setWeight($parcels_size['weight']);

                                $parcel_weight += $parcels_size['weight'];
                            }

                            $parcel_sizes[] = $paramParcelInfo;
                        }
                    }

                    if (count($parcel_sizes) == 1) {
                        $parcel_sizes_get = $parcel_sizes[0];
                        $parcel_sizes_get = $parcel_sizes_get->getWeight();

                        if (empty($parcel_sizes_get)) {
                            $parcel_sizes_set = $parcel_sizes[0];
                            $parcel_sizes_set->setWeight($data['weight']);
                            $parcel_sizes[0] = $parcel_sizes_set;
                        }
                    }

                    if ($parcel_weight) {
                        $picking->setWeightDeclared($parcel_weight);
                    }

                    $picking->setParcels($parcel_sizes);
                }

                $picking->setParcelsCount(count($parcel_sizes));

                if (!empty($data['fixed_time'])) {
                    $picking->setFixedTimeDelivery($data['fixed_time']);
                }

                $picking->setServiceTypeId($data['shipping_method']);

                if ($data['to_office'] && $data['office_id']) {
                    $picking->setOfficeToBeCalledId($data['office_id']);

                    $office = $this->getOfficeById($data['office_id']);
                } else {
                    $receiver->setAddress($receiverAddress);
                    $picking->setOfficeToBeCalledId(null);
                    $office = array();
                }

                $service = $this->getServiceById($data['shipping_method']);

                if ((empty($office) || $office->getOfficeType() != 3) && !empty($service)) {
                    if ($service->getAllowanceBackDocumentsRequest()->getValue() == 'ALLOWED') {
                        $picking->setBackDocumentsRequest($this->dataHelper->getStoreConfig('back_documents'));
                    }

                    if ($service->getAllowanceBackReceiptRequest()->getValue() == 'ALLOWED') {
                        $picking->setBackReceiptRequest($this->dataHelper->getStoreConfig('back_receipt'));
                    }
                }

                if ($this->dataHelper->getStoreConfig('from_office') && $this->dataHelper->getStoreConfig('office_id')) {
                    $picking->setWillBringToOffice(true);
                    $picking->setWillBringToOfficeId($this->dataHelper->getStoreConfig('office_id'));
                } else {
                    $picking->setWillBringToOffice(false);
                }

                $picking->setContents($data['content']);
                $picking->setPacking($data['packing']);
                $picking->setPackId(null);
                $picking->setDocuments($this->dataHelper->getStoreConfig('documents'));
                $picking->setPalletized(false);

                if (($this->dataHelper->getStoreConfig('pricing') == 'free' && $this->dataHelper->getStoreConfig('free_shipping_total') <= $data['total'])
                    || $this->dataHelper->getStoreConfig('pricing') == 'fixed'
                    || $this->dataHelper->getStoreConfig('pricing') == 'table_rate'
                    ) {
                    $payerType = \ParamCalculation::PAYER_TYPE_SENDER;
                } else {
                    $payerType = $data['payer_type'];
                }

                if ($data['abroad'] && $data['active_currency_code']) {
                    $rateToBase = $this->currencyFactory
                        ->create()
                        ->load($order->getOrderCurrencyCode())
                        ->getAnyRate($data['active_currency_code']);

                    $rateFromBase = $this->storeManager
                        ->getStore()
                        ->getBaseCurrency()
                        ->getRate($this->dataHelper->getStoreConfig('currency'));

                    if ($data['insurance']) {
                        $data['insurance_total'] = $data['insurance_total'] * ($rateToBase / $rateFromBase);
                    }

                    $data['total'] = $data['total'] * ($rateToBase / $rateFromBase);
                }

                if ($data['insurance']) {
                    if ($data['fragile']) {
                        $picking->setFragile(true);
                    } else {
                        $picking->setFragile(false);
                    }

                    $picking->setAmountInsuranceBase($data['insurance_total']);

                    $picking->setPayerTypeInsurance($payerType);
                } else {
                    $picking->setFragile(false);
                }

                $picking->setSender($sender);
                $picking->setReceiver($receiver);

                $picking->setPayerType($payerType);

                $picking->setTakingDate($data['taking_date']);

                if ($data['deffered_days']) {
                    $picking->setDeferredDeliveryWorkDays($data['deffered_days']);
                }

                if ($data['client_note']) {
                    $picking->setNoteClient($data['client_note']);
                }

                if ($this->dataHelper->getStoreConfig('pricing') == 'table_rate' && !empty($data['shipping_method_cost'])) {
                    $data['total'] += $data['shipping_method_cost'];
                }

                if ($data['cod']) {
                    $picking->setAmountCodBase($data['total']);
                } else {
                    $picking->setAmountCodBase(0);
                }

                if ($data['cod'] && $this->dataHelper->getStoreConfig('money_transfer') && !$data['abroad']) {
                    $picking->setRetMoneyTransferReqAmount($data['total']);
                    $picking->setAmountCodBase(0);
                }

                $optionBeforePayment = new \ParamOptionsBeforePayment();
                if ($data['cod'] && isset($data['option_before_payment']) && $data['option_before_payment'] != 'no_option' && (empty($office) || $office->getOfficeType() != 3)) {
                        if ($data['option_before_payment'] == 'open') {
                            $optionBeforePayment->setOpen(true);
                        } elseif ($data['option_before_payment'] == 'test') {
                            $optionBeforePayment->setTest(true);
                        }

                        $optionBeforePayment->setReturnPayerType($this->dataHelper->getStoreConfig('return_payer_type'));
                        $optionBeforePayment->setReturnServiceTypeId($this->getReturnPackageServiceTypeId($picking));
                }

                $picking->setOptionsBeforePayment($optionBeforePayment);

                if (isset($data['abroad']) && $data['abroad'] && $data['cod'] && ($data['pricing'] == 'calculator' || $data['pricing'] == 'calculator_fixed')) {
                    $picking->setIncludeShippingPriceInCod(true);
                }

                if ($this->dataHelper->getStoreConfig('return_voucher') && empty($data['abroad'])) {
                    $returnVoucher = new \ParamReturnVoucher();
                    $returnVoucher->setServiceTypeId($this->getReturnVoucherServiceTypeId($picking));
                    $returnVoucher->setPayerType($this->dataHelper->getStoreConfig('return_voucher_payer_type'));

                    $picking->setReturnVoucher($returnVoucher);
                }

                $result = $this->ePSFacade->createBillOfLading($picking);
                $parcels = $result->getGeneratedParcels();
                $parcels = $parcels[0];
                $bol['bol_id'] = $parcels->getParcelId();
                $bol['total'] = $result->getAmounts()->getTotal();
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: createBillOfLading :: ' . $e->getMessage());
            }
        }

        return $bol;
    }

    public function createPDF($bol_id)
    {
        $this->error = '';
        $pdf = '';

        if (isset($this->resultLogin)) {
            try {
                $paramPDF = new \ParamPDF();

                if ($this->dataHelper->getStoreConfig('label_printer')) {
                    $pickingParcels = $this->ePSFacade->getPickingParcels((float)$bol_id);

                    $ids = array();

                    foreach ($pickingParcels as $parcel) {
                        $ids[] = $parcel->getParcelId();
                    }

                    $paramPDF->setIds($ids);
                    $paramPDF->setType(\ParamPDF::PARAM_PDF_TYPE_LBL);
                } else {
                    $paramPDF->setIds((float)$bol_id);
                    $paramPDF->setType(\ParamPDF::PARAM_PDF_TYPE_BOL);
                }

                $paramPDF->setIncludeAutoPrintJS(true);

                $pdf = $this->ePSFacade->createPDF($paramPDF);
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: createPDF :: ' . $e->getMessage());
            }
        }

        return $pdf;
    }

    public function getAllowedDaysForTaking($data)
    {
        $this->error = '';
        $firstAvailableDate = '';

        if (isset($this->resultLogin)) {
            try {
                if ($this->dataHelper->getStoreConfig('from_office') && $this->dataHelper->getStoreConfig('office_id')) {
                    $senderSiteId = null;
                    $senderOfficeId = $this->dataHelper->getStoreConfig('office_id');
                } else {
                    $resultClientData = $this->ePSFacade->getClientById($this->resultLogin->getClientId());
                    $senderSiteId = $resultClientData->getAddress()->getSiteId();
                    $senderOfficeId = null;
                }

                $takingTime = $this->ePSFacade->getAllowedDaysForTaking($data['shipping_method_id'], $senderSiteId, $senderOfficeId, $data['taking_date']);

                if ($takingTime) {
                    $firstAvailableDate = $takingTime[0];
                }
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: getAllowedDaysForTaking :: ' . $e->getMessage());
            }
        }

        return $firstAvailableDate;
    }

    public function cancelBol($bol_id)
    {
        $this->error = '';

        if (isset($this->resultLogin)) {
            try {
                $this->ePSFacade->invalidatePicking((float)$bol_id);
            } catch (\Exception $e) {
                $this->error = $e->getMessage();
                $this->_logger->debug('Speedy :: cancelBol :: ' . $e->getMessage());
            }
        }
    }

    public function getResultLogin()
    {
        return $this->resultLogin;
    }

    public function getError()
    {
        return $this->error;
    }
}
