<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Model\Carrier;

use Extensa\Speedy\Helper\Data as dataHelper;
use Extensa\Speedy\Helper\Speedy as speedyHelper;
use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Shipping\Model\Rate\Result;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\App\Area;
use Test\Helper;

class Speedy extends \Magento\Shipping\Model\Carrier\AbstractCarrier implements
    \Magento\Shipping\Model\Carrier\CarrierInterface
{
    protected $_code = 'extensaspeedy';

    protected $_code_method = 'speedy_';

    protected $_rateResultFactory;

    protected $_rateMethodFactory;

    /**
     * @var dataHelper
     */
    protected $_dataHelper;

    /**
     * @var speedyHelper
     */
    protected $_speedyHelper;

    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    protected $_productRepository;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfiguration;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $_state;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $_product;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    protected $_searchCriteriaBuilder;

    /**
     * @var \Extensa\Speedy\Model\TablerateRepository
     */
    protected $_tablerateRepository;

    /**
     * Speedy constructor.
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory
     * @param \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory
     * @param Extensa\Speedy\Helper\Data $dataHelper
     * @param Extensa\Speedy\Helper\Speedy $speedyHelper
     * @param \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfiguration
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\App\State $state
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Extensa\Speedy\Model\TablerateRepository $tablerateRepository
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory,
        \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory,
        dataHelper $dataHelper,
        speedyHelper $speedyHelper,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfiguration,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\App\State $state,
        \Magento\Catalog\Model\Product $product,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Extensa\Speedy\Model\TablerateRepository $tablerateRepository,
        array $data = []
    ) {
        $this->_rateResultFactory = $rateResultFactory;
        $this->_rateMethodFactory = $rateMethodFactory;
        $this->_dataHelper = $dataHelper;
        $this->_speedyHelper = $speedyHelper;
        $this->_productRepository = $productRepository;
        $this->_scopeConfiguration = $scopeConfiguration;
        $this->_checkoutSession = $checkoutSession;
        $this->_storeManager = $storeManager;
        $this->_state = $state;
        $this->_product = $product;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->_tablerateRepository = $tablerateRepository;

        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);
    }

    /**
     * {@inheritdoc}
     */
    public function collectRates(RateRequest $request)
    {
        if (!$this->getConfigFlag('active')) {
            return false;
        }

        $result = $this->_rateResultFactory->create();
        $createDefaultMethodRate = true;
        $error = false;

        if ($request->getDestCountryId() != $this->_checkoutSession->getExtensaSpeedyCountry()['country_id']) {
            $this->_checkoutSession->unsExtensaSpeedy();
        }

        if ($this->_checkoutSession->hasExtensaSpeedy()) {
            $session = $this->_checkoutSession->getExtensaSpeedy();
            $allItems = $request->getAllItems();

            $currentCurrencyCode = $this->_storeManager->getStore()->getCurrentCurrencyCode();
            $currencyRate = $this->_storeManager->getStore()->getBaseCurrency()->getRate(
                $this->getConfigData('currency')
            );

            if (!$currencyRate) {
                $currencyRate = 1;
            }

            $total = round(($request->getPackageValueWithDiscount() * $currencyRate), 2);

            $weight = 0;

            foreach ($allItems as $item) {
                $item_weight = (float)$item->getWeight();
                if (!empty($item_weight)) {
                    $weight += ($item->getWeight() * $item->getQty());
                } else {
                    $weight += ($this->getConfigData('default_weight') * $item->getQty());
                }
            }

            if ($this->getConfigData('documents') && (float)$weight > 0.25) {
                $weight = 0.25;
            }

            $session['total'] = $total;
            $session['weight'] = $weight;
            $session['client_id'] = $this->getConfigData('client_id');
            $session['convertion_to_win1251'] = $this->getConfigData('convertion_to_win1251');
            $session['fixed_time'] = null;

            if ($this->getConfigData('taking_date')) {
                $session['taking_date'] = strtotime('+' . (int)$this->getConfigData('taking_date') . ' day', mktime(9, 0, 0));
            } else {
                $session['taking_date'] = time();
            }

            if (!$this->_scopeConfiguration->getValue('payment/cashondelivery/active', ScopeInterface::SCOPE_STORE)) {
                $session['cod'] = false;
            }

            $session['parcels_size'] = array(
                1 => array(
                    'weight' => '',
                    'width'  => '',
                    'height' => '',
                    'depth'  => '',
                )
            );

            $countProducts = 0;
            $parcel_size = $this->_dataHelper->getParcelSize(1);
            $products = array();

            if (!$session['abroad']) {
                foreach ($allItems as $item) {
                    if (!$item->getFileDownloadParams()) {
                        $countProducts += $item->getQty();
                        $sizes = $this->_getSpeedyQuantityDimention($item->getData('product_id'), $item->getQty());

                        if (!empty($sizes) || $this->_speedyHasQuantityDimention($item->getData('product_id'))) {
                            $sizes['quantity'] = $item->getQty();
                            $products[] = $sizes;

                            if (!empty($sizes['size'])) {
                                $parcel_size = $this->_dataHelper->compareParcelSizes($parcel_size, $sizes['size']);
                            }
                        } else {
                            $no_parcel_size = true;
                        }
                    }
                }

                $weight_size = $this->_getSpeedyWeightDimention($session['weight'], $countProducts);

                if (!empty($products) && empty($no_parcel_size)) {
                    for ($i = 1;$i <= count($this->_dataHelper->getParcelSizeList()); $i++) {
                        $parcel_full = 0;

                        foreach ($products as $product) {
                            if (empty($product['sizes'])) {
                                $parcel_size = '';
                                break 2;
                            }
                            $parcel_full += $product['quantity'] / $product['sizes'][$parcel_size];
                        }

                        if ($parcel_full > 1) {
                            $next_size = $this->_dataHelper->getNextParcelSize($parcel_size);

                            if ($next_size) {
                                $parcel_size = $next_size;
                            } else {
                                $parcel_size = '';
                                break;
                            }
                        } else {
                            break;
                        }
                    }
                } elseif ($weight_size) {
                    $size_compare = $this->_calculateSize($products, $parcel_size);

                    if ($size_compare) {
                        $parcel_size = $this->_dataHelper->compareParcelSizes($size_compare, $weight_size);
                    } else {
                        $parcel_size = $weight_size;
                    }
                } elseif ($this->getConfigData('min_package_dimention')) {
                    $size_compare = $this->_calculateSize($products, $parcel_size);

                    if ($size_compare) {
                        $parcel_size = $this->_dataHelper->compareParcelSizes($size_compare, $this->getConfigData('min_package_dimention'));
                    } else {
                        $parcel_size = $this->getConfigData('min_package_dimention');
                    }
                } else {
                    $parcel_size = '';
                }
            } else {
                $parcel_size = '';
            }

            $session['parcel_size'] = $parcel_size;

            $methods = $this->_speedyHelper->calculate($session);
            $services = $this->_speedyHelper->getServices();
            $methods_count = 0;

            if (!$this->_speedyHelper->getError()) {
                foreach ($methods as $method_id => $method) {
                       if ($method->getServiceTypeId() == 500 && !empty($session['parcel_size'])) {
                            $method_title = $this->getConfigData('name') . ' - ' . $services[$method->getServiceTypeId()] . ' (' . $session['parcel_size'] . ')';
                        } else {
                            $method_title = $this->getConfigData('name') . ' - ' . $services[$method->getServiceTypeId()];
                        }
                        if (($this->getConfigData('pricing') == 'free')
                            && ($total >= $this->getConfigData('free_shipping_total'))
                            && ($method->getServiceTypeId() == $this->getConfigData('free_method_city')
                                || $method->getServiceTypeId() == $this->getConfigData('free_method_intercity')
                                || in_array($method->getServiceTypeId(), $this->_dataHelper->getStoreConfigArray('free_method_international'))
                                )
                            ) {
                            $method_total = 0;
                        } elseif ($this->getConfigData('pricing') == 'fixed') {
                            $method_total = $this->getConfigData('fixed_price');
                        } elseif ($this->getConfigData('pricing') == 'table_rate') {
                            $filter = $this->_searchCriteriaBuilder
                                ->addFilter('service_id', (int)$method->getServiceTypeId(), 'eq')
                                ->addFilter('take_from_office', (int)$session['to_office'], 'eq')
                                ->addFilter('weight', (float)$session['weight'], 'gteq')
                                ->addFilter('order_total', (float)$session['total'], 'gteq');

                            if (!empty($session['fixed_time_cb'])) {
                                $filter = $filter->addFilter('fixed_time_delivery', (int)$session['fixed_time_cb'], 'eq')
                                    ->create();
                            } else {
                                $filter = $filter->addFilter('fixed_time_delivery', 0, 'eq')
                                    ->create();
                            }

                            $tableRate = $this->_tablerateRepository->getFirstResult($filter);

                            if ($tableRate->getPriceWithoutVat() == null) {
                                continue;
                            } else {
                                $method_total = $tableRate->getPriceWithoutVat();
                            }
                        } else {
                            $method_total = $method->getResultInfo()->getAmounts()->getTotal();

                            if ($this->getConfigData('pricing') == 'calculator_fixed') {
                                $method_total += $this->getConfigData('fixed_price');
                            }
                        }

                        $rateMethod = $this->_rateMethodFactory->create();

                        $rateMethod->setCarrier($this->_code);
                        $rateMethod->setCarrierTitle($this->getConfigData('title'));

                        $rateMethod->setMethod($this->_code_method . $method->getServiceTypeId());
                        $rateMethod->setMethodTitle($method_title);

                        $rateMethod->setPrice((float)$method_total / $currencyRate);

                        // add this rate to the result
                        $result->append($rateMethod);

                        $methods_count++;
                    }

                if ($methods_count) {
                    $session['service'] = $method->getServiceTypeId();
                    $createDefaultMethodRate = false;
                } else {
                    $error = true;
                    $session['error'] = __('There are no available Speedy methods for this address!');;
                }
            } else {
                $error = true;
                $session['error'] = $this->_dataHelper->getError();
            }

            $this->_checkoutSession->setExtensaSpeedy($session);
        }

        $this->_checkoutSession->setExtensaSpeedyCountry(array('country_id' => $request->getDestCountryId()));

        // Create default method rate if there is no speedy services && there is no errors
        if ($createDefaultMethodRate) {
            if ($error) {
                $error = $this->_rateErrorFactory->create();
                $error->setCarrier($this->_code);
                $error->setCarrierTitle($this->getConfigData('title'));
                $error->setErrorMessage($session['error']);
                $result->append($error);
            } else {
                $rateMethod = $this->_rateMethodFactory->create();

                $rateMethod->setCarrier($this->_code);
                $rateMethod->setCarrierTitle($this->getConfigData('title'));
                $rateMethod->setMethod($this->_code);
                $rateMethod->setPrice(0);
                $rateMethod->setCost(0);

                $result->append($rateMethod);
            }
        }

        return $result;
    }

    private function _calculateSize($products, $size_compare)
    {
        if(!empty($products)) {
            for ($i = 1;$i <= count($this->_dataHelper->getParcelSizeList()); $i++) {
                $parcel_full = 0;

                foreach ($products as $product) {
                    if (!empty($product['sizes'])) {
                        $parcel_full += $product['quantity'] / $product['sizes'][$size_compare];
                    }
                }

                if ($parcel_full > 1) {
                    $next_size = $this->_dataHelper->getNextParcelSize($parcel_size);

                    if ($next_size) {
                        $parcel_size = $next_size;
                    } else {
                        $parcel_size = '';
                        break;
                    }
                } else {
                    break;
                }
            }
        }

        return $size_compare;
    }

    private function _speedyHasQuantityDimention($product_id)
    {
        $_product = $this->_productRepository->getById($product_id);

        foreach ($this->_dataHelper->getParcelSizeList() as $size) {
            if ($_product->getData('speedy_quantity_dimension_' . $size['size'])) {
                return true;
            }
        }

        return false;
    }

    private function _getSpeedyQuantityDimention($product_id, $product_quantity)
    {
        $_product = $this->_productRepository->getById($product_id);

        $sizes = [];

        foreach ($this->_dataHelper->getParcelSizeList() as $size) {
            $sizes[$size['size']] = $_product->getData('speedy_quantity_dimension_' . $size['size']);
        }

        if ($sizes) {
            uasort($sizes, array($this, '_cmp'));

            foreach ($sizes as $size => $quantity) {
                if ($quantity >= $product_quantity) {
                    return array(
                        'size'     => $size,
                        'sizes'    => $sizes,
                    );
                }
            }
        }

        return false;
    }

    private function _getSpeedyWeightDimention($weight, $product_quantity)
    {
        $data = json_decode($this->getConfigData('weight_dimensions'));

        foreach($data as $d) {
            if ($d->WEIGHT >= $weight) {
                unset($d->WEIGHT);
                $sizes = json_decode(json_encode($d), true);
            }
        }

        if (!empty($sizes)) {
            uasort($sizes, array(get_class($this), '_cmp'));

            foreach ($sizes as $size => $quantity) {
                if ($quantity >= $product_quantity) {
                    return $size;
                }
            }
        }

        return false;
    }

    // sorts the array by quantity without deleting the keys
    private function _cmp($a, $b)
    {
        if ($a == $b) {
            return 0;
        }

        return ($a < $b) ? -1 : 1;
    }

    /**
     * @return array
     */
    public function getAllowedMethods()
    {
        return [$this->_code => $this->getConfigData('name')];
    }
}
