<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Model\Checkout;

use Magento\Store\Model\ScopeInterface;
use Extensa\Speedy\Api\AddressRepositoryInterface;
use Magento\Checkout\Model\ConfigProviderInterface;

class ConfigProvider implements ConfigProviderInterface
{
    /**
     * @var \Extensa\Speedy\Helper\Data
     */
    protected $dataHelper;

    /**
     * @var \Extensa\Speedy\Helper\Speedy
     */
    protected $speedyHelper;

    /**
     * @var \Extensa\Speedy\Model\Carrier\Speedy
     */
    protected $carrierSpeedy;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfiguration;

    /**
     * @var AddressRepositoryInterface
     */
    protected $addressRepository;
    /**
     * @var AddressFactory
     */
    protected $addressFactory;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * @var \Magento\Framework\Locale\Resolver
     */
    protected $store;

    /**
     * ConfigProvider constructor.
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Helper\Speedy $speedyHelper
     * @param \Extensa\Speedy\Model\Carrier\Speedy $carrierSpeedy
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfiguration
     * @param \Extensa\Speedy\Api\AddressRepositoryInterface $addressRepository
     * @param \Extensa\Speedy\Api\Data\AddressInterfaceFactory $addressFactory
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Framework\Locale\Resolver $store
     */
    public function __construct(
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Extensa\Speedy\Helper\Speedy $speedyHelper,
        \Extensa\Speedy\Model\Carrier\Speedy $carrierSpeedy,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfiguration,
        \Extensa\Speedy\Api\AddressRepositoryInterface $addressRepository,
        \Extensa\Speedy\Api\Data\AddressInterfaceFactory $addressFactory,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Framework\Locale\Resolver $store
    ) {
        $this->dataHelper = $dataHelper;
        $this->speedyHelper = $speedyHelper;
        $this->carrierSpeedy = $carrierSpeedy;
        $this->scopeConfiguration = $scopeConfiguration;
        $this->addressRepository = $addressRepository;
        $this->addressFactory = $addressFactory;
        $this->checkoutSession = $checkoutSession;
        $this->store = $store;
    }

    /**
     * {@inheritdoc}
     */
    public function getConfig()
    {
        $result = [];
        $speedyOrderData = [];
        $session = [];

        if ($this->checkoutSession->hasExtensaSpeedy()) {
            $session = $this->checkoutSession->getExtensaSpeedy();
        }

        if ($session) {
            $speedyOrderData = $session;
        } else {
            if ($this->checkoutSession->getQuote()->getShippingAddress()->getCustomerAddressId()) {
                try {
                    $receiverAddress = $this->addressRepository->getById(
                        $this->checkoutSession->getQuote()->getShippingAddress()->getCustomerAddressId()
                    );

                    if ($receiverAddress->getId()) {
                        $speedyOrderData = $receiverAddress->getData();
                    }
                } catch (\Exception $e) {
                }
            }

            if (!$speedyOrderData) {
                $receiverAddress = $this->checkoutSession->getQuote()->getShippingAddress();

                $speedyOrderData['postcode'] = '';
                $speedyOrderData['required_postcode'] = '';
                $speedyOrderData['country'] = '';
                $speedyOrderData['country_iso_2'] = $this->checkoutSession->getQuote()->getShippingAddress()->getCountryId();
                $speedyOrderData['country_id'] = '';
                $speedyOrderData['country_nomenclature'] = '';
                $speedyOrderData['active_currency_code'] = '';
                $speedyOrderData['abroad'] = '';
                $speedyOrderData['state'] = '';
                $speedyOrderData['state_id'] = '';
                $speedyOrderData['required_state'] = '';
                $speedyOrderData['city'] = '';
                $speedyOrderData['city_id'] = '';
                $speedyOrderData['city_nomenclature'] = '';
                $speedyOrderData['to_office'] = '';
                $speedyOrderData['office_id'] = '';
                $speedyOrderData['quarter'] = '';
                $speedyOrderData['quarter_id'] = '';
                $speedyOrderData['street'] = '';
                $speedyOrderData['street_id'] = '';
                $speedyOrderData['street_no'] = '';
                $speedyOrderData['block_no'] = '';
                $speedyOrderData['entrance_no'] = '';
                $speedyOrderData['floor_no'] = '';
                $speedyOrderData['apartment_no'] = '';
                $speedyOrderData['note'] = '';
                $speedyOrderData['address_1'] = '';
                $speedyOrderData['address_2'] = '';
            }

            $speedyOrderData['fixed_time_hour'] = '';
            $speedyOrderData['fixed_time_min'] = '';
            $speedyOrderData['fixed_time_cb'] = '';
        }

        $speedyOrderData['cod_active'] = (bool)$this->scopeConfiguration->getValue('payment/cashondelivery/active', ScopeInterface::SCOPE_STORE);
        if ($speedyOrderData['cod_active']) {
            $speedyOrderData['cod'] = true;
        } else {
            $speedyOrderData['cod'] = false;
        }

        if ($speedyOrderData['city_id']) {
            $locale = $speedyOrderData['abroad'] ? 'EN' : 'BG';

            $speedyOrderData['offices'] = $this->speedyHelper->getOffices(null, $speedyOrderData['city_id'], $locale);
        } else {
            $speedyOrderData['offices'] = array();
        }

        $speedyOrderData['error'] = $this->speedyHelper->getError();

        $result['extensa_speedy']['order_data'] = $speedyOrderData;
        $result['extensa_speedy']['fixed_time'] = $this->dataHelper->getStoreConfigFlag('fixed_time');
        $result['extensa_speedy']['store_locale'] = $this->store->getLocale();
        $result['extensa_speedy']['default_country'] = $this->scopeConfiguration->getValue('general/country/default', ScopeInterface::SCOPE_STORE);

        $result['extensa_speedy']['autocomplete_city_url'] = $this->dataHelper->getAutocompleteCityUrl();
        $result['extensa_speedy']['autocomplete_quarter_url'] = $this->dataHelper->getAutocompleteQuarterUrl();
        $result['extensa_speedy']['autocomplete_street_url'] = $this->dataHelper->getAutocompleteStreetUrl();
        $result['extensa_speedy']['autocomplete_state_url'] = $this->dataHelper->getAutocompleteStateUrl();
        $result['extensa_speedy']['get_country_url'] = $this->dataHelper->getCountryUrl();
        $result['extensa_speedy']['get_office_url'] = $this->dataHelper->getOfficeUrl();
        $result['extensa_speedy']['validate_url'] = $this->dataHelper->getValidateUrl();

        return $result;
    }
}
