<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Model\System\Config\Backend;

use \Magento\Config\Model\Config\Backend\File;

class Tablerate extends File
{
    /**
     * @var \Magento\Framework\File\Csv
     */
    protected $csvProcessor;

    /**
     * @var \Extensa\Speedy\Helper\Data
     */
    protected $dataHelper;

    /**
     * @var \Extensa\Speedy\Model\ResourceModel\Tablerate\CollectionFactory
     */
    protected $speedyTablerateCollectionFactory;

    /**
     * @var \Extensa\Speedy\Api\Data\TablerateInterfaceFactory $speedyTablerateFactory
     */
    protected $speedyTablerateFactory;

    /**
     * @var \Extensa\Speedy\Api\TablerateRepositoryInterface $tablerateRepository
     */
    protected $tablerateRepository;

    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $config
     * @param \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList
     * @param \Magento\MediaStorage\Model\File\UploaderFactory $uploaderFactory
     * @param \Magento\Config\Model\Config\Backend\File\RequestData\RequestDataInterface $requestData
     * @param \Magento\Framework\Filesystem $filesystem
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb $resourceCollection
     * @param \Magento\Framework\File\Csv $csvProcessor
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Model\ResourceModel\Tablerate\CollectionFactory $speedyTablerateCollectionFactory
     * @param \Extensa\Speedy\Api\Data\TablerateInterfaceFactory $speedyTablerateFactory
     * @param \Extensa\Speedy\Api\TablerateRepositoryInterface $tablerateRepository
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\App\Config\ScopeConfigInterface $config,
        \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList,
        \Magento\MediaStorage\Model\File\UploaderFactory $uploaderFactory,
        \Magento\Config\Model\Config\Backend\File\RequestData\RequestDataInterface $requestData,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        \Magento\Framework\File\Csv $csvProcessor,
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Extensa\Speedy\Model\ResourceModel\Tablerate\CollectionFactory $speedyTablerateCollectionFactory,
        \Extensa\Speedy\Api\Data\TablerateInterfaceFactory $speedyTablerateFactory,
        \Extensa\Speedy\Api\TablerateRepositoryInterface $tablerateRepository,
        array $data = []
    ) {
        $this->csvProcessor = $csvProcessor;
        $this->dataHelper = $dataHelper;
        $this->speedyTablerateCollectionFactory = $speedyTablerateCollectionFactory;
        $this->speedyTablerateFactory = $speedyTablerateFactory;
        $this->tablerateRepository = $tablerateRepository;

        parent::__construct(
            $context,
            $registry,
            $config,
            $cacheTypeList,
            $uploaderFactory,
            $requestData,
            $filesystem,
            $resource,
            $resourceCollection,
            $data
        );
    }

    /**
     * Save uploaded file before saving config value
     *
     * @return \Extensa\Speedy\Model\Config\Backend\Tablerate
     * @throws \Magento\Framework\Exception\ValidatorException
     */
    public function beforeSave()
    {
        $file = $this->getFileData();
        if (!empty($file)) {
            $this->speedyTablerateCollectionFactory->create()->walk('delete');

            $importTableRateRawData = $this->csvProcessor->getData($file['tmp_name']);

            $colums = $importTableRateRawData[0];

            unset($importTableRateRawData[0]);

            foreach ($importTableRateRawData as $data) {
                $speedyTablerate = $this->speedyTablerateFactory->create();

                $insert = [];

                foreach ($data as $key => $value) {
                    if (!empty($colums[$key])) {
                        $tableRateColumn = $this->dataHelper->getTableRateColumn($colums[$key]);

                        if (!empty($tableRateColumn) && is_numeric($value)) {
                            $insert[$tableRateColumn] = $value;
                        } else {
                            throw new \Magento\Framework\Exception\ValidatorException(__('Invalid numbers in CSV file.Value is not numeric!'));
                            break 2;
                        }
                    } else {
                        throw new \Magento\Framework\Exception\ValidatorException(__('Invalid numbers in CSV file.Use dots for delimiter!'));
                        break 2;
                    }
                }

               $speedyTablerate->setData($insert);

              $this->tablerateRepository->save($speedyTablerate);
            }
       }

        $this->setValue('');

        return $this;
    }

    /**
     * @return array
     */
    public function getAllowedExtensions() {
        return ['csv'];
    }
}