<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Observer\Checkout;

use Magento\Framework\App\Area;

class SubmitAllAfter implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var \Extensa\Speedy\Helper\Data
     */
    protected $dataHelper;

    /**
     * @var \Extensa\Speedy\Api\OrderRepositoryInterface
     */
    protected $speedyOrderRepository;

    /**
     * @var \Extensa\Speedy\Api\Data\OrderInterfaceFactory
     */
    protected $speedyOrderFactory;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * @var \Magento\Backend\Model\Session
     */
    protected $backendSession;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $state;

    /**
     * SubmitAllAfter constructor.
     * @param \Extensa\Speedy\Helper\Data $dataHelper
     * @param \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository
     * @param \Extensa\Speedy\Api\Data\OrderInterfaceFactory $speedyOrderFactory
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Backend\Model\Session $backendSession
     * @param \Magento\Framework\App\State $state
     */
    public function __construct (
        \Extensa\Speedy\Helper\Data $dataHelper,
        \Extensa\Speedy\Api\OrderRepositoryInterface $speedyOrderRepository,
        \Extensa\Speedy\Api\Data\OrderInterfaceFactory $speedyOrderFactory,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Backend\Model\Session $backendSession,
        \Magento\Framework\App\State $state
    ) {
        $this->dataHelper = $dataHelper;
        $this->speedyOrderRepository = $speedyOrderRepository;
        $this->speedyOrderFactory = $speedyOrderFactory;
        $this->checkoutSession = $checkoutSession;
        $this->backendSession = $backendSession;
        $this->state = $state;
    }

    /**
     * Execute observer
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(
        \Magento\Framework\Event\Observer $observer
    ) {
        if ($this->checkoutSession->hasExtensaSpeedy()) {
            $session = $this->checkoutSession->getExtensaSpeedy();
            $session['pricing'] = $this->dataHelper->getStoreConfig('pricing');

            $order = $observer->getEvent()->getOrder();

            if (strpos($order->getShippingMethod(), 'extensaspeedy') !== false) {
                $speedyOrder = $this->speedyOrderFactory->create();
                $speedyOrder
                    ->setDataColumn(serialize($session))
                    ->setOrderId($order->getId());

                $this->speedyOrderRepository->save($speedyOrder);
                $this->checkoutSession->unsExtensaSpeedy();
            }
        }
    }
}
