<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Setup;

use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

class InstallSchema implements InstallSchemaInterface
{
    /**
     * @param SchemaSetupInterface $setup
     * @param ModuleContextInterface $context
     */
    public function install(
        SchemaSetupInterface $setup,
        ModuleContextInterface $context
    ) {
        $installer = $setup;
        $installer->startSetup();

        /**
         * Creating table extensa_speedy_address
         */
        $extensaSpeedyAddressTableName = 'extensa_speedy_address';

        if (!$installer->getConnection()->isTableExists($extensaSpeedyAddressTableName)) {
            $extensaSpeedyAddressTableName = $installer->getConnection()
                ->newTable($installer->getTable($extensaSpeedyAddressTableName))
                ->addColumn(
                    'address_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'nullable'  => false,
                    ]
                )
                ->addColumn(
                    'customer_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'nullable'  => false,
                    ]
                )
                ->addColumn(
                    'postcode',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    10,
                    [
                        'nullable'  => false,
                        'default'   => '',
                    ]
                )
                ->addColumn(
                    'required_postcode',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    1,
                    [
                        'nullable'  => false,
                        'default'   => 0
                    ]
                )
                ->addColumn(
                    'country',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'country_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'nullable'  => false,
                        'default'   => 100,
                    ],
                    '100 - България'
                )
                ->addColumn(
                    'country_nomenclature',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    15,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'abroad',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    1,
                    [
                        'nullable'  => false,
                        'default'   => 0
                    ]
                )
                ->addColumn(
                    'state',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'state_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'nullable'  => false,
                    ]
                )
                ->addColumn(
                    'required_state',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    1,
                    [
                        'nullable'  => false,
                        'default'   => 0
                    ]
                )
                ->addColumn(
                    'city',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => '',
                    ]
                )
                ->addColumn(
                    'city_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'nullable'  => false,
                        'default'   => 0
                    ]
                )
                ->addColumn(
                    'city_nomenclature',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    15,
                    [
                        'nullable'  => false,
                        'default'   => '',
                    ]
                )
                ->addColumn(
                    'take_office',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    1,
                    [
                        'nullable'  => false,
                        'default'   => 0
                    ]
                )
                ->addColumn(
                    'office_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    10,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'quarter',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'quarter_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'nullable'  => false,
                        'default'   => 0
                    ]
                )
                ->addColumn(
                    'street',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'street_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'nullable'  => false,
                        'default'   => 0
                    ]
                )
                ->addColumn(
                    'street_no',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'block_no',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'entrance_no',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'floor_no',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'apartment_no',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'note',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'address_1',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'address_2',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addIndex(
                    $installer->getIdxName(
                        $extensaSpeedyAddressTableName,
                        ['address_id'],
                        \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_INDEX
                    ),
                    ['address_id'],
                    ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_INDEX]
                )
                ->addIndex(
                    $installer->getIdxName(
                        $extensaSpeedyAddressTableName,
                        ['customer_id'],
                        \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_INDEX
                    ),
                    ['customer_id'],
                    ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_INDEX]
                );

            $installer->getConnection()->createTable($extensaSpeedyAddressTableName);
        }

        /**
         * Creating table extensa_speedy_order
         */
        $extensaSpeedyOrderTableName = 'extensa_speedy_order';

        if (!$installer->getConnection()->isTableExists($extensaSpeedyOrderTableName)) {
            $extensaSpeedyOrderTableName = $installer->getConnection()
                ->newTable($installer->getTable($extensaSpeedyOrderTableName))
                ->addColumn(
                    'speedy_order_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'identity'  => true,
                        'nullable'  => false,
                        'primary'   => true,
                    ]
                )
                ->addColumn(
                    'order_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'nullable'  => false,
                        'default'   => 0,
                    ]
                )
                ->addColumn(
                    'bol_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    15,
                    [
                        'nullable'  => false,
                    ]
                )
                ->addColumn(
                    'data',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    null,
                    [
                        'nullable'  => false,
                    ]
                )
                ->addColumn(
                    'date_created',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    null,
                    [
                        'nullable'  => false,
                    ]
                )
                ->addColumn(
                    'courier',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    1,
                    [
                        'nullable'  => false,
                        'default'   => 0
                    ]
                )
                ->addIndex(
                    $installer->getIdxName(
                        $extensaSpeedyOrderTableName,
                        ['order_id'],
                        \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_INDEX
                    ),
                    ['order_id'],
                    ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_INDEX]
                );

            $installer->getConnection()->createTable($extensaSpeedyOrderTableName);
        }

        /**
         * Creating table extensa_speedy_table_rate
         */
        $extensaSpeedyTableRateTableName = 'extensa_speedy_table_rate';

        if (!$installer->getConnection()->isTableExists($extensaSpeedyTableRateTableName)) {
            $extensaSpeedyTableRateTableName = $installer->getConnection()
                ->newTable($installer->getTable($extensaSpeedyTableRateTableName))
                ->addColumn(
                    'table_rate_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'identity'  => true,
                        'nullable'  => false,
                        'primary'   => true,
                    ]
                )
                ->addColumn(
                    'service_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'nullable'  => false,
                    ]
                )
                ->addColumn(
                    'take_from_office',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    1,
                    [
                        'nullable'  => false,
                        'default'   => 0
                    ]
                )
                ->addColumn(
                    'weight',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    15,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'order_total',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    15,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'price_without_vat',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    15,
                    [
                        'nullable'  => false,
                        'default'   => ''
                    ]
                )
                ->addColumn(
                    'fixed_time_delivery',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    1,
                    [
                        'nullable'  => false,
                        'default'   => 0
                    ]
                )
                ->addIndex(
                    $installer->getIdxName(
                        $extensaSpeedyTableRateTableName,
                        ['table_rate_id'],
                        \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_INDEX
                    ),
                    ['table_rate_id'],
                    ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_INDEX]
                );

            $installer->getConnection()->createTable($extensaSpeedyTableRateTableName);
        }

        $installer->endSetup();
    }
}
