<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Ui\DataProvider\Product\Form\Modifier;

use Magento\Catalog\Ui\DataProvider\Product\Form\Modifier\General;
use Magento\Catalog\Model\Locator\LocatorInterface;
use Magento\Framework\Stdlib\ArrayManager;

class Quantitydimension extends \Magento\Catalog\Ui\DataProvider\Product\Form\Modifier\General
{
    /**
     * @var \Extensa\Speedy\Helper\Data
     */
    protected $dataHelper;

    /**
     * @param Magento\Catalog\Model\Locator\LocatorInterface $locator
     * @param Magento\Framework\Stdlib\ArrayManager $arrayManager
     * @param Extensa\Speedy\Helper\Data $dataHelper
     */
    public function __construct(
        \Magento\Catalog\Model\Locator\LocatorInterface $locator,
        \Magento\Framework\Stdlib\ArrayManager $arrayManager,
        \Extensa\Speedy\Helper\Data $dataHelper
    ) {
        $this->dataHelper = $dataHelper;

        parent::__construct(
            $locator,
            $arrayManager
        );
    }

    public function modifyMeta(array $meta)
    {
        $meta = $this->getAttributes($meta);

        return $meta;
    }

    private function getAttributes($meta)
    {
        $parcel_sizes = $this->dataHelper->getParcelSizeList();
        $paths = array();

        foreach ($parcel_sizes as $parcel_size) {
            $paths[$parcel_size['size']] = $this->arrayManager->findPath(
                'speedy_quantity_dimension_' . $parcel_size['size'],
                $meta,
                null,
                'children'
            );
        }

        if ($paths) {
            $attributeGroup = 'speedy_quantity_dimension';

            $meta = $this->arrayManager->merge(
                reset($paths),
                $meta,
                $this->getCustomAttributeStructure($paths, $parcel_sizes, $meta)
            );
            $meta = $this->arrayManager->set(
                $this->arrayManager->slicePath(reset($paths), 0, -3)
                . '/' . $attributeGroup,
                $meta,
                $this->arrayManager->get(reset($paths), $meta)
            );

            foreach ($paths as $path) {
                $meta = $this->arrayManager->remove(
                    $this->arrayManager->slicePath($path, 0, -2),
                    $meta
                );
            }
        }

        return $meta;
    }

    private function getCustomAttributeStructure($paths, $parcel_sizes, $meta) {
        $return['arguments'] = [
            'data' => [
                'config' => [
                    'formElement' => 'container',
                    'componentType' => 'container',
                    'breakLine' => false,
                    'label' => __('Speedy POST - Maximum number of units of product filling the package'),
                    'additionalClasses' => '',
                    'visible' => true,
                    'required' => false,
                    'sortOrder' => $this->arrayManager->get(reset($paths) . '/arguments/data/config/sortOrder', $meta),
                    'component' => 'Magento_Ui/js/form/components/group',
                ]
            ]
        ];

        foreach ($parcel_sizes as $parcel_size) {
            $return['children']['speedy_quantity_dimension_' . $parcel_size['size']] = [
                'arguments' => [
                    'data' => [
                        'config' => [
                            'dataType' => \Magento\Ui\Component\Form\Element\DataType\Text::NAME,
                            'componentType' => \Magento\Ui\Component\Form\Field::NAME,
                            'formElement' => 'input',
                            'visible' => true,
                            'required' => false,
                            'label' => $parcel_size['size'] . ' ' . '(' . $parcel_size['height'] . ' x ' . $parcel_size['width'] . ' x ' . $parcel_size['depth'] . ')',
                            'additionalClasses' => 'admin__field-horizontal_field',
                        ]
                    ]
                ]
            ];
        }

        return $return;
    }
}
