define(
    [
        'jquery',
        'uiComponent',
        'ko',
        'Magento_Checkout/js/checkout-data',
        'Magento_Checkout/js/model/quote',
        'Magento_Checkout/js/model/shipping-service',
        'Magento_Checkout/js/model/shipping-rate-registry',
        'Magento_Checkout/js/model/shipping-rate-processor/customer-address',
        'Magento_Checkout/js/model/shipping-rate-processor/new-address',
        'underscore',
        'mage/translate',
        'jquery/validate', // Jquery Validation Library
        'mage/validation',
    ],
    function(
        $,
        Component,
        ko,
        checkout_data,
        quote,
        shippingService,
        rateRegistry,
        customerAddressProcessor,
        newAddressProcessor,
        _,
        $t
    ) {
        'use strict';

        var extensa_speedy = window.checkoutConfig.extensa_speedy;

        // Create a "class" that represents an option
        var Option = function(id, caption) {
            this.id = id;
            this.caption = caption;
        };

        var store_locale = extensa_speedy.store_locale;

        var speedy_city = extensa_speedy.order_data.city;
        var speedy_quarter = extensa_speedy.order_data.quarter;
        var speedy_street = extensa_speedy.order_data.street;
        var speedy_state = extensa_speedy.order_data.state;

        var self;

        return Component.extend({
            defaults: {
                template: 'Extensa_Speedy/checkout/shipping/extensa_speedy_form'
            },
            htmlId: 'extensa_speedy',
            carrier_code: 'extensaspeedy',
            extensa_speedy_visible_methods: [],

            initialize: function () {
                self = this;
                this._super(); //you must call super on components or they will not render

                for (var i = 10; i <= 17; i++) {
                    self.extensa_speedy_fixed_time_hours.push(
                        new Option(i, i)
                    );
                }

                // show/hide form
                quote.shippingMethod.subscribe(this.getIsAbroad.bind(this), null, 'change');
                quote.shippingMethod.subscribe(this.showSpeedyError.bind(this), null, 'change');

                quote.shippingAddress.subscribe(this.getCountry.bind(this), null, 'change');

                self.extensa_speedy_city_id.subscribe(this.extensaSpeedyClearChosenMethodValue.bind(this), null, 'change');
                self.extensa_speedy_cod.subscribe(this.extensaSpeedyClearChosenMethodValue.bind(this), null, 'change');
                self.extensa_speedy_to_office.subscribe(this.extensaSpeedyClearChosenMethodValue.bind(this), null, 'change');
                self.extensa_speedy_fixed_time_cb.subscribe(this.extensaSpeedyClearChosenMethodValue.bind(this), null, 'change');

                $(document).on('keyup', '#' + self.htmlId + '-form input', function() {
                    if (self.extensa_speedy_abroad() && $(this).val().match(/[а-яА-я]/)) {
                        $(this).val('');
                        alert($t('Please, use only latin characters!'));
                    }
                });

                $(document).on('change', '#' + self.htmlId + '-form input, #' + self.htmlId + '-form select', function() {
                    self.extensaSpeedyClearChosenMethodValue();

                    $('input[value^=' + self.carrier_code + '_speedy_]').each(function() {
                        $(this).parent().parent().remove();
                    });
                });

                shippingService.getShippingRates().subscribe(function (rates) {
                    self.extensa_speedy_visible_methods = [];
                    _.each(rates, function (rate) {
                        if (rate && rate['carrier_code'] !== undefined && rate['carrier_code'].indexOf(self.carrier_code) !== -1) {
                            self.extensa_speedy_visible_methods.push(rate['method_code']);
                        }
                    });
                });

                var offices = extensa_speedy.order_data.offices;

                Object.keys(offices).forEach(function(key) {
                    self.extensa_speedy_offices.push(
                        new Option(offices[key]['id'], offices[key]['label'])
                    );
                });
            },
            yesnoOptions: ko.observableArray([
                new Option(1, $t('Yes')),
                new Option(0, $t('No'))
            ]),
            sendOptions: ko.observableArray([
                new Option(1, $t('To office')),
                new Option(0, $t('To door'))
            ]),

            getIsAbroad: function (selectedCarrier) {
                if (self.getCountryId() == 'BG') {
                    this.extensa_speedy_abroad(0);
                } else {
                    this.extensa_speedy_abroad(1);
                }
            },

            getCountryId: function() {
                if (extensa_speedy.default_country) {
                    quote.shippingAddress().countryId = extensa_speedy.default_country;
                    extensa_speedy.default_country = '';
                }

                return quote.shippingAddress().countryId;
            },

            getLocale: function () {
                if (self.getCountryId() != 'BG') {
                    return 'EN';
                } else {
                    return store_locale.split('_')[0].toUpperCase();
                }
            },

            getCountry: function (selectedCarrier) {
                var thisObj = this;
                var country_iso = self.getCountryId();

                $.ajax({
                    url: extensa_speedy.get_country_url,
                    type: 'POST',
                    data: {
                        iso_code_2: country_iso,
                        locale:     self.getLocale()
                    },
                    dataType: 'json',
                    showLoader: true,
                    success: function(json) {
                        if (!json.success) {
                            alert(json.message);
                        } else {
                            if (json.country.hasOwnProperty(country_iso)) {
                                if (json.country.id != thisObj.extensa_speedy_country_id()) {
                                    thisObj.extensaSpeedyClearAddress();
                                }

                                thisObj.extensa_speedy_country_iso_2(json.country[country_iso].iso_code_2);
                                thisObj.extensa_speedy_country_id(json.country[country_iso].id);
                                thisObj.extensa_speedy_country(json.country[country_iso].name);
                                thisObj.extensa_speedy_country_nomenclature(json.country[country_iso].nomenclature);
                                thisObj.extensa_speedy_required_state(json.country[country_iso].required_state);
                                thisObj.extensa_speedy_active_currency_code(json.country[country_iso].active_currency_code);

                                thisObj.getIsAbroad(thisObj);

                                if (!self.extensa_speedy_abroad()) {
                                    jQuery("#extensa_speedy_postcode").addClass('speedy-disabled');
                                } else {
                                    jQuery("#extensa_speedy_postcode").removeClass('speedy-disabled');
                                }

                                if (!json.country[country_iso].active_currency_code) {
                                    thisObj.extensa_speedy_cod(0);
                                } else {
                                    thisObj.extensa_speedy_cod(1);
                                }
                            }
                        }
                    }
                });
            },

            showSpeedyError: function (selectedCarrier) {
                if (selectedCarrier && selectedCarrier.method_code !== undefined && selectedCarrier.carrier_code !== undefined && selectedCarrier.carrier_code.indexOf(this.carrier_code) !== -1 && selectedCarrier.method_code == 'error') {
                    self.extensa_speedy_error($t('For this order can not be calculated price. Please contact the administrators of the store!'));
                    self.extensaSpeedyClearChosenMethodValue();
                } else {
                    self.extensa_speedy_error('');
                }
            },

            extensaSpeedyClearChosenMethodValue: function () {
                self.extensa_speedy_shipping_method('');
            },

            setFixedTime: function () {
                self.extensa_speedy_fixed_time_mins([]);

                if (self.extensa_speedy_fixed_time_hour() == 10) {
                    var min_fixed_time_mins = 30;
                } else {
                    var min_fixed_time_mins = 0;
                }

                if (self.extensa_speedy_fixed_time_hour() == 17) {
                    var max_fixed_time_mins = 30;
                } else {
                    var max_fixed_time_mins = 59;
                }

                for (var i = min_fixed_time_mins; i <= max_fixed_time_mins; i++) {
                    var iStr = i.toString();

                    if (iStr.length < 2) {
                        var fixed_time_min = '0' + i;
                    } else {
                        var fixed_time_min = i;
                    }

                    self.extensa_speedy_fixed_time_mins.push(
                        new Option(fixed_time_min, fixed_time_min)
                    );
                }
            },

            extensaSpeedyClearAddress: function () {
                self.extensa_speedy_postcode('');
                self.extensa_speedy_required_postcode('');
                self.extensa_speedy_country('');
                self.extensa_speedy_country_id('');
                self.extensa_speedy_country_nomenclature('');
                self.extensa_speedy_abroad('');
                self.extensa_speedy_state('');
                self.extensa_speedy_state_id('');
                self.extensa_speedy_required_state('');
                self.extensa_speedy_city('');
                self.extensa_speedy_city_id('');
                self.extensa_speedy_city_nomenclature('');
                self.extensa_speedy_to_office('');
                self.extensa_speedy_office_id(0);
                self.extensa_speedy_quarter('');
                self.extensa_speedy_quarter_id('');
                self.extensa_speedy_street('');
                self.extensa_speedy_street_id('');
                self.extensa_speedy_street_no('');
                self.extensa_speedy_block_no('');
                self.extensa_speedy_entrance_no('');
                self.extensa_speedy_floor_no('');
                self.extensa_speedy_apartment_no('');
                self.extensa_speedy_note('');
                self.extensa_speedy_address_1('');
                self.extensa_speedy_address_2('');
                self.extensa_speedy_fixed_time_hour('');
                self.extensa_speedy_fixed_time_min('');
                self.extensa_speedy_fixed_time_cb('');
                self.extensa_speedy_error('');
                self.extensa_speedy_offices([]);
            },

            extensaSpeedyCity: function (element) {
                $(element).autocomplete({
                    delay: 500,
                    minLength: 1,
                    autoFocus: true,
                    'source': function (request, response) {
                        $.ajax({
                            url: extensa_speedy.autocomplete_city_url,
                            type: 'POST',
                            data: {
                                city: request.term,
                                country_id: self.extensa_speedy_country_id,
                                locale:     self.getLocale()
                            },
                            dataType: 'json',
                            showLoader: true,
                            success: function(json) {
                                if (!json.success) {
                                    alert(json.message);
                                } else {
                                    if ($('#extensa_speedy_city').is(":focus")) {
                                        response($.map(json.items, function(item) {
                                            return {
                                                id:           item['city_id'],
                                                label:        item['label'],
                                                value:        item['value'],
                                                postcode:     item['postcode'],
                                                region:       item['nomenclature']
                                            }
                                        }));
                                    }
                                }
                            }
                        });
                    },
                    'select': function (event, ui) {
                        if (ui.item) {
                            speedy_city = ui.item.value;

                            self.extensa_speedy_city(speedy_city);
                            self.extensa_speedy_city_id(ui.item.id);
                            self.extensa_speedy_postcode(ui.item.postcode);
                            self.extensa_speedy_city_nomenclature(ui.item.nomenclature);
                            self.extensa_speedy_quarter('');
                            self.extensa_speedy_quarter_id('');
                            self.extensa_speedy_street('');
                            self.extensa_speedy_street_id('');
                            self.extensa_speedy_street_no('');
                            self.extensa_speedy_block_no('');
                            self.extensa_speedy_entrance_no('');
                            self.extensa_speedy_floor_no('');
                            self.extensa_speedy_apartment_no('');
                            self.extensa_speedy_note('');
                            self.extensa_speedy_offices([]);
                            self.extensa_speedy_office_id(0);

                           $.ajax({
                                url: extensa_speedy.get_office_url,
                                type: 'POST',
                                data: {
                                    city_id: ui.item.id,
                                    locale:  self.getLocale()
                                },
                                dataType: 'json',
                                showLoader: true,
                                success: function(json) {
                                    if (!json.success) {
                                        alert(json.message);
                                    } else {
                                        var response = json.items;

                                        Object.keys(response).forEach(function(key) {
                                            self.extensa_speedy_offices.push(
                                                new Option(response[key]['id'], response[key]['label'])
                                            );
                                        });
                                    }
                                }
                            });
                        }
                    },
                    'change': function(event, ui) {
                        if(!ui.item && self.extensa_speedy_country_nomenclature == 'FULL') {
                            self.extensa_speedy_city('');
                            self.extensa_speedy_city_id('');
                            self.extensa_speedy_city_nomenclature('');
                            self.extensa_speedy_postcode('');
                            self.extensa_speedy_office_id(0);
                            self.extensa_speedy_quarter('');
                            self.extensa_speedy_quarter_id('');
                            self.extensa_speedy_street('');
                            self.extensa_speedy_street_id('');
                            self.extensa_speedy_street_no('');
                            self.extensa_speedy_offices([]);
                        }
                    }
                });

                $(element).blur(function() {
                    if (self.extensa_speedy_city() != speedy_city) {
                        if (!self.extensa_speedy_abroad || (self.extensa_speedy_abroad && (self.extensa_speedy_country_nomenclature == 'FULL'))) {
                            self.extensa_speedy_city('');
                        }

                        self.extensa_speedy_city_id('');
                        self.extensa_speedy_city_nomenclature('');
                        self.extensa_speedy_postcode('');
                        self.extensa_speedy_office_id(0);
                        self.extensa_speedy_quarter('');
                        self.extensa_speedy_quarter_id('');
                        self.extensa_speedy_street('');
                        self.extensa_speedy_street_id('');
                        self.extensa_speedy_street_no('');
                        self.extensa_speedy_offices([]);
                    }
                });
            },

            extensaSpeedyQuarter: function (element) {
                $(element).autocomplete({
                    delay: 500,
                    minLength: 1,
                    autoFocus: true,
                    'source': function (request, response) {
                        $.ajax({
                            url: extensa_speedy.autocomplete_quarter_url,
                            type: 'POST',
                            data: {
                                name:       request.term,
                                city_id:    self.extensa_speedy_city_id,
                                locale:     self.getLocale()
                            },
                            dataType: 'json',
                            showLoader: true,
                            success: function(json) {
                                if (!json.success) {
                                    alert(json.message);
                                } else {
                                    if ($('#extensa_speedy_quarter').is(":focus")) {
                                        response($.map(json.items, function(item) {
                                            return {
                                                id:             item['id'],
                                                label:          item['label'],
                                                value:          item['value']
                                            }
                                        }));
                                    }
                                }
                            }
                        });
                    },
                    'select': function (event, ui) {
                        if (ui.item) {
                            speedy_quarter = ui.item.value;
                            self.extensa_speedy_quarter(speedy_quarter);
                            self.extensa_speedy_quarter_id(ui.item.id);
                        }
                    },
                    'change': function(event, ui) {
                        if(!ui.item) {
                            self.extensa_speedy_quarter('');
                            self.extensa_speedy_quarter_id('');
                        }
                    }
                });

                $(element).blur(function() {
                    if (self.extensa_speedy_quarter() != speedy_quarter) {
                        self.extensa_speedy_quarter('');
                        self.extensa_speedy_quarter_id('');
                    }
                });
            },

            extensaSpeedyStreet: function (element) {
                $(element).autocomplete({
                    delay: 500,
                    minLength: 1,
                    autoFocus: true,
                    'source': function (request, response) {
                        $.ajax({
                            url: extensa_speedy.autocomplete_street_url,
                            type: 'POST',
                            data: {
                                name:       request.term,
                                city_id:    self.extensa_speedy_city_id,
                                locale:     self.getLocale()
                            },
                            dataType: 'json',
                            showLoader: true,
                            success: function(json) {
                                if (!json.success) {
                                    alert(json.message);
                                } else {
                                    if ($('#extensa_speedy_street').is(":focus")) {
                                        response($.map(json.items, function(item) {
                                            return {
                                                id:             item['id'],
                                                label:          item['label'],
                                                value:          item['value']
                                            }
                                        }));
                                    }
                                }
                            }
                        });
                    },
                    'select': function (event, ui) {
                        if (ui.item) {
                            speedy_street = ui.item.value;
                            self.extensa_speedy_street(speedy_street);
                            self.extensa_speedy_street_id(ui.item.id);
                        }
                    },
                    'change': function(event, ui) {
                        if(!ui.item) {
                            self.extensa_speedy_street('');
                            self.extensa_speedy_street_id('');
                        }
                    }
                });

                $(element).blur(function() {
                    if (self.extensa_speedy_street() != speedy_street) {
                        self.extensa_speedy_street('');
                        self.extensa_speedy_street_id('');
                    }
                });
            },

            extensaSpeedyState: function (element) {
                $(element).autocomplete({
                    delay: 500,
                    minLength: 1,
                    autoFocus: true,
                    'source': function (request, response) {
                        $.ajax({
                            url: extensa_speedy.autocomplete_state_url,
                            type: 'POST',
                            data: {
                                name:       request.term,
                                country_id: self.extensa_speedy_country_id
                            },
                            dataType: 'json',
                            showLoader: true,
                            success: function(json) {
                                if (!json.success) {
                                    alert(json.message);
                                } else {
                                    if ($('#extensa_speedy_state').is(":focus")) {
                                        response($.map(json.items, function(item) {
                                            return {
                                                id:             item['id'],
                                                label:          item['name'],
                                                value:          item['name']
                                            }
                                        }));
                                    }
                                }
                            }
                        });
                    },
                    'select': function (event, ui) {
                        if (ui.item) {
                            speedy_state = ui.item.value;
                            self.extensa_speedy_state(speedy_state);
                            self.extensa_speedy_state_id(ui.item.id);
                        }
                    },
                    'change': function(event, ui) {
                        if(!ui.item) {
                            self.extensa_speedy_state('');
                            self.extensa_speedy_state_id('');
                        }
                    }
                });

                $(element).blur(function() {
                    if (self.extensa_speedy_state() != speedy_state) {
                        self.extensa_speedy_state('');
                        self.extensa_speedy_state_id('');
                    }
                });
            },

            validateForm: function (form) {
                return $(form).validation() && $(form).validation('isValid');
            },

            extensaSpeedyCalculatePrice: function() {
                $('#extensa_speedy_shipping_method').hide();

                if (this.validateForm($('#' + self.htmlId + '-form').closest('form'))) {
                    var extensa_speedy_disabled = $('#' + self.htmlId + '-form input:disabled');
                    $('#' + self.htmlId + '-form input:disabled').removeAttr('disabled');

                    if (quote.shippingMethod()) {
                        var shipping_method = quote.shippingMethod()['carrier_code'] + '_' + quote.shippingMethod()['method_code'];
                    } else {
                        var shipping_method = self.carrier_code + '_' + self.carrier_code;
                    }

                    $.ajax({
                        url: extensa_speedy.validate_url,
                        type: 'POST',
                        data: $('#' + self.htmlId + '-form input, #' + self.htmlId + '-form select').serialize() + '&shipping_method=' + shipping_method,
                        dataType: 'json',
                        showLoader: true,
                        success: function(json) {
                            if (!json.success) {
                                alert(json.message);
                            } else {
                                extensa_speedy_disabled.attr('disabled', true);

                                var address = quote.shippingAddress();
                                // clearing cached rates to retrieve new ones
                                rateRegistry.set(address.getCacheKey(), null);
                                var type = quote.shippingAddress().getType();
                                if (type && type == 'customer-address') {
                                    customerAddressProcessor.getRates(address);
                                } else {
                                    newAddressProcessor.getRates(address);
                                }
                                self.extensa_speedy_shipping_method(shipping_method);
                            }
                        }
                    });
                }

                $('#extensa_speedy_shipping_method').show();

                return false;
            },

            extensa_speedy_fixed_time_hours: ko.observableArray(),
            extensa_speedy_fixed_time_mins: ko.observableArray(),
            extensa_speedy_offices: ko.observableArray(),
            extensa_speedy_shipping_method: ko.observableArray(),

            extensa_speedy_postcode: ko.observable(extensa_speedy.order_data.postcode),
            extensa_speedy_required_postcode: ko.observable(extensa_speedy.order_data.required_postcode),
            extensa_speedy_country: ko.observable(extensa_speedy.order_data.country),
            extensa_speedy_country_iso_2: ko.observable(extensa_speedy.order_data.country_iso_2),
            extensa_speedy_country_id: ko.observable(extensa_speedy.order_data.country_id),
            extensa_speedy_country_nomenclature: ko.observable(extensa_speedy.order_data.country_nomenclature),
            extensa_speedy_active_currency_code: ko.observable(extensa_speedy.order_data.active_currency_code),
            extensa_speedy_abroad: ko.observable(extensa_speedy.order_data.abroad),
            extensa_speedy_state: ko.observable(extensa_speedy.order_data.state),
            extensa_speedy_state_id: ko.observable(extensa_speedy.order_data.state_id),
            extensa_speedy_required_state: ko.observable(extensa_speedy.order_data.required_state),
            extensa_speedy_city: ko.observable(extensa_speedy.order_data.city),
            extensa_speedy_city_id: ko.observable(extensa_speedy.order_data.city_id),
            extensa_speedy_city_nomenclature: ko.observable(extensa_speedy.order_data.city_nomenclature),
            extensa_speedy_to_office: ko.observable(extensa_speedy.order_data.to_office),
            extensa_speedy_office_id: ko.observable(extensa_speedy.order_data.office_id),
            extensa_speedy_quarter: ko.observable(extensa_speedy.order_data.quarter),
            extensa_speedy_quarter_id: ko.observable(extensa_speedy.order_data.quarter_id),
            extensa_speedy_street: ko.observable(extensa_speedy.order_data.street),
            extensa_speedy_street_id: ko.observable(extensa_speedy.order_data.street_id),
            extensa_speedy_street_no: ko.observable(extensa_speedy.order_data.street_no),
            extensa_speedy_block_no: ko.observable(extensa_speedy.order_data.block_no),
            extensa_speedy_entrance_no: ko.observable(extensa_speedy.order_data.entrance_no),
            extensa_speedy_floor_no: ko.observable(extensa_speedy.order_data.floor_no),
            extensa_speedy_apartment_no: ko.observable(extensa_speedy.order_data.apartment_no),
            extensa_speedy_note: ko.observable(extensa_speedy.order_data.note),
            extensa_speedy_address_1: ko.observable(extensa_speedy.order_data.address_1),
            extensa_speedy_address_2: ko.observable(extensa_speedy.order_data.address_2),
            extensa_speedy_fixed_time_hour: ko.observable(extensa_speedy.order_data.fixed_time_hour),
            extensa_speedy_fixed_time_min: ko.observable(extensa_speedy.order_data.fixed_time_min),
            extensa_speedy_fixed_time_cb: ko.observable(extensa_speedy.order_data.fixed_time_cb),
            extensa_speedy_cod_active: ko.observable(extensa_speedy.order_data.cod_active),
            extensa_speedy_cod: ko.observable(extensa_speedy.order_data.cod),

            extensa_speedy_error: ko.observable(extensa_speedy.error),
            extensa_speedy_fixed_time: ko.observable(extensa_speedy.fixed_time),
            extensa_speedy_shipping_method: ko.observable(extensa_speedy.order_data.shipping_method),

            extensaSpeedyOrderData: extensa_speedy.order_data
        });
    }
);
